@@ -1,52 +1,55 @@
+#! /bin/sh
+# the next line restarts with tclsh \
+exec tclsh "$0" ${1+"$@"}
+
 # rdate.tcl - Copyright (C) 2003 Pat Thoyts <patthoyts@users.sourceforge.net>
 #
-# NAME
-#  rdate - set the system's date from a remote host
-#
-# SYNOPSIS
-#  rdate [-psa] [-ut] host
-#
-# DESCRIPTION
-#  Rdate displays and sets the local date and time from the host name or ad-
-#  dress given as the argument. It uses the RFC868 protocol which is usually
-#  implemented as a built-in service of inetd(8).
-#
-#  Available options:
-#
-#  -p      Do not set, just print the remote time
-#
-##  -s      Do not print the time.
-##
-##  -a      Use the adjtime(2) call to gradually skew the local time to the
-##          remote time rather than just hopping.
-#
-#  -u      Use UDP
-#
-#  -t      Use TCP
+# This is a sample implementation of the rdate(8) utility written using the
+# tcllib ntp time package.
 #
 # -------------------------------------------------------------------------
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id$
+# $Id: rdate.tcl,v 1.4 2009/01/30 04:18:14 andreas_kupries Exp $
 
 package require time;                   # tcllib 1.4
 
+proc usage {} {
+    set s {usage: rdate [-psa] [-utS] host
+  -p    Do not set, just print the remote time.
+  -s    Do not print the time. [NOT IMPLEMENTED]
+  -a    Use the adjtime(2) call to gradually skew the local time to the
+        remote time instead of just jumping. [NOT IMPLEMENTED]
+  -u    Use UDP (default if available)
+  -t    Use TCP
+  -S    Use SNTP protocol (RFC 2030) (default is TIME, RFC 868)
+}
+    return $s
+}
+
+proc Error {message} {
+    puts stderr $message
+    exit 1
+}
+
 proc rdate {args} {
     # process the command line options.
-    array set opts {-p 0 -s 0 -a 0 -t 0 -u x}
+    array set opts {-p 0 -s 0 -a 0 -t 0 -u x -S 0}
     while {[string match -* [set option [lindex $args 0]]]} {
         switch -exact -- $option {
             -p { set opts(-p) 1 }
             -u { set opts(-t) 0 }
             -t { set opts(-t) 1 }
-            -s { return -code error "not implemented: use rdate(8)" }
-            -a { return -code error "not implemented: use rdate(8)" }
+            -s { Error "not implemented: use rdate(8)" }
+            -a { Error "not implemented: use rdate(8)" }
+            -S { set opts(-S) 1 }
+            -T { set opts(-S) 0 }
             -- { ::time::Pop args; break }
             default {
                 set err [join [lsort [array names opts -*]] ", "]
-                return -code error "bad option $option: must be $err"
+                Error "bad option $option: must be $err"
             }
         }
         ::time::Pop args
@@ -54,12 +57,15 @@ proc rdate {args} {
 
     # Check that we have a host to talk to.
     if {[llength $args] != 1} {
-        return -code error "wrong \# args: "
+        Error [usage]
     }
     set host [lindex $args 0]
 
     # Construct the time command - optionally force the protocol to tcp
     set cmd ::time::gettime
+    if {$opts(-S)} {
+        set cmd ::time::getsntp
+    }
     if {$opts(-t)} {
         lappend cmd -protocol tcp
     }
@@ -76,7 +82,7 @@ proc rdate {args} {
     } else {
         set msg [::time::error $tok]
         ::time::cleanup $tok
-        return -code error $msg 
+        Error $msg 
     }
 
     # Display the time.
@@ -84,9 +90,10 @@ proc rdate {args} {
         puts [clock format $t]
     }
 
-    return
+    return 0
 }
 
 if {! $tcl_interactive} {
     eval rdate $argv
 }
+
