@@ -22,8 +22,8 @@
 package require Tcl 8.2
 
 namespace eval ::aes {
-    variable version 1.0.0
-    variable rcsid {$Id: aes.tcl,v 1.5 2005/09/04 17:15:56 patthoyts Exp $}
+    variable version 1.1
+    variable rcsid {$Id: aes.tcl,v 1.7 2010/07/06 19:39:00 andreas_kupries Exp $}
     variable uid ; if {![info exists uid]} { set uid 0 }
 
     namespace export {aes}
@@ -146,7 +146,7 @@ proc ::aes::EncryptBlock {Key block} {
             return -code error "invalid initialization vector: must be 16 bytes"
         }
         for {set n 0} {$n < 4} {incr n} {
-            lappend data2 [expr {[lindex $data $n] ^ [lindex $iv $n]}]
+            lappend data2 [expr {0xffffffff & ([lindex $data $n] ^ [lindex $iv $n])}]
         }
         set data $data2
     }
@@ -156,6 +156,15 @@ proc ::aes::EncryptBlock {Key block} {
         set data [AddRoundKey $Key $n [MixColumns [ShiftRows [SubBytes $data]]]]
     }
     set data [AddRoundKey $Key $n [ShiftRows [SubBytes $data]]]
+
+    # Bug 2993029:
+    # Force all elements of data into the 32bit range.
+    set res {}
+    foreach d $data {
+        lappend res [expr {$d & 0xffffffff}]
+    }
+    set data $res
+    
     return [set state(I) [binary format I4 $data]]
 }
 
@@ -178,11 +187,20 @@ proc ::aes::DecryptBlock {Key block} {
             return -code error "invalid initialization vector: must be 16 bytes"
         }
         for {set n 0} {$n < 4} {incr n} {
-            lappend data2 [expr {[lindex $data $n] ^ [lindex $iv $n]}]
+            lappend data2 [expr {0xffffffff & ([lindex $data $n] ^ [lindex $iv $n])}]
         }
         set data $data2
+    } else {
+        # Bug 2993029:
+        # Force all elements of data into the 32bit range.
+        # The trimming we see above only happens for CBC mode.
+        set res {}
+        foreach d $data {
+            lappend res [expr {$d & 0xffffffff}]
+        }
+        set data $res
     }
-    
+
     set state(I) $block
     return [binary format I4 $data]
 }
@@ -485,7 +503,7 @@ proc ::aes::Pad {data blocksize {fill \0}} {
 }
 
 proc ::aes::Pop {varname {nth 0}} {
-    upvar $varname args
+    upvar 1 $varname args
     set r [lindex $args $nth]
     set args [lreplace $args $nth $nth]
     return $r
@@ -501,7 +519,7 @@ proc ::aes::aes {args} {
     set opts(-iv) [string repeat \0 16]
     set modes {ecb cbc}
     set dirs {encrypt decrypt}
-    while {[string match -* [set option [lindex $args 0]]]} {
+    while {([llength $args] > 1) && [string match -* [set option [lindex $args 0]]]} {
         switch -exact -- $option {
             -mode      { set opts(-mode) [SetOneOf $modes [Pop args 1]] }
             -dir       { set opts(-dir) [SetOneOf $dirs [Pop args 1]] }
@@ -555,7 +573,7 @@ proc ::aes::aes {args} {
         }
 
         set Key [Init $opts(-mode) $opts(-key) $opts(-iv)]
-        upvar $Key state
+        upvar 1 $Key state
         set state(reading) 1
         if {[string equal $opts(-dir) "encrypt"]} {
             set state(cmd) Encrypt
