@@ -45,7 +45,7 @@ proc testsNeedTcltest {version} {
     # testsuite. If the minimum is not met by the loaded package we
     # forcibly bail out of the testsuite calling the command. The
     # command has to be called after loading the utilities. The only
-    # command allowed to come before it is 'textNeedTcl' above.
+    # command allowed to come before it is 'testNeedTcl' above.
 
     # Note that this command will try to load a suitable version of
     # Tcltest if the package has not been loaded yet.
@@ -77,7 +77,16 @@ proc testsNeed {name version} {
     # testsuite calling the command. The command has to be called
     # immediately after loading the utilities.
 
-    if {[package vsatisfies [package provide $name] $version]} return
+    if {[catch {
+	package require $name $version
+    }]} {
+	puts "    Aborting the tests found in \"[file tail [info script]]\""
+	puts "    Requiring at least $name $version, package not found."
+
+	return -code return
+    }
+
+    if {[package vsatisfies [package present $name] $version]} return
 
     puts "    Aborting the tests found in \"[file tail [info script]]\""
     puts "    Requiring at least $name $version, have [package present $name]."
@@ -197,6 +206,9 @@ proc InitializeTclTest {} {
     ::tcltest::testConstraint tcl8.4minus \
 	[expr {![package vsatisfies [package provide Tcl] 8.5]}]
 
+    ::tcltest::testConstraint tcl8.5minus \
+	[expr {![package vsatisfies [package provide Tcl] 8.6]}]
+
     # ### ### ### ######### ######### #########
     ## Cross-version code for the generation of the error messages created
     ## by Tcl procedures when called with the wrong number of arguments,
