@@ -29,7 +29,7 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id: dns.tcl,v 1.35 2007/08/26 00:44:34 patthoyts Exp $
+# $Id: dns.tcl,v 1.36 2008/11/22 12:28:54 mic42 Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 package require logger;                 # tcllib 1.3
@@ -38,8 +38,8 @@ package require uri::urn;               # tcllib 1.2
 package require ip;                     # tcllib 1.7
 
 namespace eval ::dns {
-    variable version 1.3.2
-    variable rcsid {$Id: dns.tcl,v 1.35 2007/08/26 00:44:34 patthoyts Exp $}
+    variable version 1.3.3
+    variable rcsid {$Id: dns.tcl,v 1.36 2008/11/22 12:28:54 mic42 Exp $}
 
     namespace export configure resolve name address cname \
         status reset wait cleanup errorcode
@@ -913,7 +913,7 @@ proc ::dns::Flags {token {varname {}}} {
     set flags(authoritative)      [expr {($hdr & 0x0400) >> 10}]
     set flags(truncated)          [expr {($hdr & 0x0200) >> 9}]
     set flags(recursion_desired)  [expr {($hdr & 0x0100) >> 8}]
-    set flafs(recursion_allowed)  [expr {($hdr & 0x0080) >> 7}]
+    set flags(recursion_allowed)  [expr {($hdr & 0x0080) >> 7}]
     set flags(errorcode)          [expr {($hdr & 0x000F)}]
 
     return [array get flags]
