@@ -1,9 +1,9 @@
 #temporary home until this gets cleaned up for export to tcllib ip module
+# $Id: ipMore.tcl,v 1.4 2006/01/22 00:27:22 andreas_kupries Exp $
 
 
 ##Library Header
 #
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2005 Cisco Systems, Inc.
 #
 # Name:
@@ -100,7 +100,6 @@ interp alias {} ::ip::IpHostFromPrefix    {} ::ip::ipHostFromPrefix
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -139,7 +138,7 @@ proc ip::prefixToNativeTcl {prefix} {
     set plist {}
     foreach p $prefix {
 	set newPrefix [ip::toHex [ip::prefix $p]]
-	if {[set mask [ip::mask $p]] eq ""} {
+	if {[string equal [set mask [ip::mask $p]] ""]} {
 	    set newMask 0xffffffff
 	} else {
 	    set newMask [format "0x%08x" [ip::maskToInt $mask]]
@@ -151,7 +150,6 @@ proc ip::prefixToNativeTcl {prefix} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -213,7 +211,7 @@ proc ::ip::nativeToPrefix {nativeList args} {
     if {[llength [lindex $nativeList 0]]==1} {set pList 0; set nativeList [list $nativeList]}
     foreach native $nativeList {
 	lassign $native ip mask
-	if {$mask eq ""} {set mask 32}
+	if {[string equal $mask ""]} {set mask 32}
 	set pString ""
 	append pString [ip::ToString [binary format I [expr {$ip}]]]
 	append pString  "/"
@@ -227,7 +225,6 @@ proc ::ip::nativeToPrefix {nativeList args} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -279,7 +276,6 @@ proc ::ip::intToString {int args} {
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -320,7 +316,6 @@ proc ::ip::toInteger {ip} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -361,7 +356,6 @@ proc ::ip::toHex {ip} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -399,15 +393,15 @@ proc ::ip::toHex {ip} {
 
 proc ::ip::maskToInt {mask} {
     if {[string is integer -strict $mask]} {
-        set maskInt [expr {(0xFFFFFFFF << (32 - $mask)) & 0xFFFFFFFF}]
+        set maskInt [expr {(0xFFFFFFFF << (32 - $mask))}]
     } else {
         binary scan [Normalize4 $mask] I maskInt
     }
-    return [expr wide([format "0x%08x" $maskInt])]
+    set maskInt [expr {$maskInt & 0xFFFFFFFF}]
+    return [format %u $maskInt]
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -474,7 +468,6 @@ proc ::ip::broadcastAddress {prefix args} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -559,7 +552,6 @@ proc ::ip::maskToLength {mask args} {
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -610,7 +602,6 @@ proc ::ip::lengthToMask {masklen args} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -672,7 +663,6 @@ proc ::ip::nextNet {prefix mask args} {
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -733,7 +723,6 @@ proc ::ip::isOverlap {ip args} {
 #optimized overlap, that accepts native format
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -830,7 +819,6 @@ proc ::ip::isOverlapNativeTcl {args} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -872,7 +860,6 @@ proc ::ip::ipToLayer2Multicast { ipaddr } {
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -999,7 +986,6 @@ proc ::ip::ipHostFromPrefix { prefix args } {
 
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -1054,7 +1040,7 @@ proc ::ip::reduceToAggregates { prefixList } {
 	    set overLap [ip::isOverlapNative [lindex $current 0] [lindex $current 1] $remaining]
 	    if {$overLap} {
 		#there was a overlap find out which prefix has a the smaller mask, and keep that one
-		if {[lindex $current 1] > [lindex [lindex $remaining [expr {$overLap -1}] 1]]} {
+		if {[lindex $current 1] > [lindex [lindex $remaining [expr {$overLap -1}]] 1]} {
 		    #current has more restrictive mask, throw that prefix away
 		    # keep other prefix
 		    lappend nonOverLapping [lindex $remaining [expr {$overLap -1}]]
@@ -1076,7 +1062,6 @@ proc ::ip::reduceToAggregates { prefixList } {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -1132,7 +1117,7 @@ proc ::ip::longestPrefixMatch { ipaddr prefixList args} {
 	set dotConv 1
     }
     #sort so that most specific prefix is in the front
-    if {[llength [lindex $prefixList 0 1]]} {
+    if {[llength [lindex [lindex $prefixList 0] 1]]} {
 	set prefixList [lsort -decreasing -integer -index 1 $prefixList]
     } else {
 	set prefixList [list $prefixList]
@@ -1149,7 +1134,6 @@ proc ::ip::longestPrefixMatch { ipaddr prefixList args} {
 }
 
 ##Procedure Header
-# $Id: ipMore.tcl,v 1.1 2005/09/30 18:03:58 andreas_kupries Exp $
 # Copyright (c) 2004 Cisco Systems, Inc.
 #
 # Name:
@@ -1186,18 +1170,37 @@ proc ::ip::longestPrefixMatch { ipaddr prefixList args} {
 # End of Header
 #            ip address in <ipprefix> format, dotted form, or integer form
 
-proc ip::cmpDotIP {ipaddr1 ipaddr2} {
-    # convert dotted to decimal
-    set ipInt1 [::ip::toHex $ipaddr1]
-    set ipInt2 [::ip::toHex $ipaddr2]
-    #ipMore::log::debug "$ipInt1 $ipInt2"
-    if { $ipInt1 < $ipInt2}  {
-	return -1
-    } elseif {$ipInt1 >$ipInt2 } {
-	return 1
-    } else {
+if {![package vsatisfies [package provide Tcl] 8.4]} {
+    # 8.3+
+    proc ip::cmpDotIP {ipaddr1 ipaddr2} {
+	# convert dotted to list of integers
+	set ipaddr1 [split $ipaddr1 .]
+	set ipaddr2 [split $ipaddr2 .]
+	foreach a $ipaddr1 b $ipaddr2 {
+	    #ipMore::log::debug "$ipInt1 $ipInt2"
+	    if { $a < $b}  {
+		return -1
+	    } elseif {$a >$b} {
+		return 1
+	    }
+	}
 	return 0
     }
+} else {
+    # 8.4+
+    proc ip::cmpDotIP {ipaddr1 ipaddr2} {
+	# convert dotted to decimal
+	set ipInt1 [::ip::toHex $ipaddr1]
+	set ipInt2 [::ip::toHex $ipaddr2]
+	#ipMore::log::debug "$ipInt1 $ipInt2"
+	if { $ipInt1 < $ipInt2}  {
+	    return -1
+	} elseif {$ipInt1 >$ipInt2 } {
+	    return 1
+	} else {
+	    return 0
+	}
+    }
 }
 
 # Populate the array "maskLenToDotted" for fast lookups of mask to
