@@ -4,7 +4,7 @@
 # Code used inside of a checker interpreter to ensure correct usage of
 # doctools formatting commands.
 #
-# Copyright (c) 2003-2009 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2010 Andreas Kupries <andreas_kupries@sourceforge.net>
 
 # L10N
 
@@ -115,7 +115,7 @@ proc Warn {code args} {
     set msg [::msgcat::mc $code]
     foreach {off line col} [dt_where] break
     set msg [eval [linsert $args 0 format $msg]]
-    set msg "In macro at line $line, column $col:\n$msg"
+    set msg "In macro at line $line, column $col of file [dt_file]:\n$msg"
     set msg [split $msg \n]
     set prefix "DocTools Warning ($code): "
     dt_warning "$prefix[join $msg "\n$prefix"]"
@@ -126,7 +126,7 @@ proc WarnX {code args} {
     set msg [::msgcat::mc $code]
     foreach {off line col} [dt_where] break
     set msg [eval [linsert $args 0 format $msg]]
-    set msg "In macro at line $line, column $col:\n$msg"
+    set msg "In macro at line $line, column $col of file [dt_file]:\n$msg"
     set msg [split $msg \n]
     set prefix "DocTools Warning ($code): "
     dt_warning "$prefix[join $msg "\n$prefix"]"
@@ -271,6 +271,7 @@ proc vset {var args} {
 proc manpage_begin {title section version} {
     Enter manpage_begin
     if {[IsNot manpage_begin]} {Error mpbegin}
+    if {[string match {* *} $title]} {Error mptitle}
     Go header
     fmt_manpage_begin $title $section $version
 }
@@ -387,8 +388,9 @@ proc list_begin {what {hint {}}} {
     if {[LOpen] && ![LItem]} {Error nolisthdr}
     set what [LMap $what]
     if {![LValid $what]}     {Error invalidlist $what}
+    set res [fmt_list_begin $what $hint]
     LPush        $what
-    fmt_list_begin $what $hint
+    return $res
 }
 proc list_end {} {
     Enter list_end
@@ -683,6 +685,18 @@ proc uri {text {label {}}} {
 	fmt_uri $text $label
     }
 }
+proc image {text {label {}}} {
+    if {[Is done]} {Error nodonecmd}
+    # The label argument is left out when undefined so that we can
+    # control old formatters as well, if the input is not using uri
+    # labels.
+
+    if {$label == {}} {
+	fmt_image $text
+    } else {
+	fmt_image $text $label
+    }
+}
 proc manpage {text} {
     if {[Is done]} {Error nodonecmd}
     # The label argument is left out when undefined so that we can
@@ -705,4 +719,13 @@ proc term {text} {
     fmt_term $text
 }
 
+proc mdash {} {
+    if {[Is done]} {Error nodonecmd}
+    fmt_mdash $text
+}
+proc ndash {} {
+    if {[Is done]} {Error nodonecmd}
+    fmt_ndash $text
+}
+
 # -------------------------------------------------------------
