@@ -2,16 +2,23 @@
 #
 #	Implementation of doctoc objects for Tcl.
 #
-# Copyright (c) 2003-2005 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2010 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: doctoc.tcl,v 1.11 2005/09/30 05:36:39 andreas_kupries Exp $
+# RCS: @(#) $Id: doctoc.tcl,v 1.22 2010/06/08 19:13:53 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require textutil::expander
 
+# @mdgen OWNER: api_toc.tcl
+# @mdgen OWNER: checker_toc.tcl
+# @mdgen OWNER: mpformats/*.tcl
+# @mdgen OWNER: mpformats/*.msg
+# @mdgen OWNER: mpformats/toc.*
+# @mdgen OWNER: mpformats/man.macros
+
 namespace eval ::doctools {}
 namespace eval ::doctools::toc {
     # Data storage in the doctools::toc module
@@ -224,8 +231,8 @@ proc ::doctools::toc::_configure {name args} {
     if {[llength $args] == 0} {
 	# Retrieve the current configuration.
 
-	upvar ::doctools::toc::doctoc${name}::file    file
-	upvar ::doctools::toc::doctoc${name}::format  format
+	upvar #0 ::doctools::toc::doctoc${name}::file    file
+	upvar #0 ::doctools::toc::doctoc${name}::format  format
 
 	set     res [list]
 	lappend res -file       $file
@@ -237,11 +244,11 @@ proc ::doctools::toc::_configure {name args} {
 
 	switch -exact -- [lindex $args 0] {
 	    -file {
-		upvar ::doctools::toc::doctoc${name}::file file
+		upvar #0 ::doctools::toc::doctoc${name}::file file
 		return $file
 	    }
 	    -format {
-		upvar ::doctools::toc::doctoc${name}::format format
+		upvar #0 ::doctools::toc::doctoc${name}::format format
 		return $format
 	    }
 	    default {
@@ -260,14 +267,14 @@ proc ::doctools::toc::_configure {name args} {
 	foreach {option value} $args {
 	    switch -exact -- $option {
 		-file {
-		    upvar ::doctools::toc::doctoc${name}::file file
+		    upvar #0 ::doctools::toc::doctoc${name}::file file
 		    set file $value
 		}
 		-format {
 		    if {[catch {
 			set fmtfile [LookupFormat $name $value]
 			SetupFormatter $name $fmtfile
-			upvar ::doctools::toc::doctoc${name}::format format
+			upvar #0 ::doctools::toc::doctoc${name}::format format
 			set format $value
 		    } msg]} {
 			return -code error "doctools::toc::_configure: -format: $msg"
@@ -327,7 +334,7 @@ proc ::doctools::toc::_destroy {name} {
 #	None.
 
 proc ::doctools::toc::_map {name sfname afname} {
-    upvar ::doctools::toc::doctoc${name}::map map
+    upvar #0 ::doctools::toc::doctoc${name}::map map
     set map($sfname) $afname
     return
 }
@@ -345,17 +352,17 @@ proc ::doctools::toc::_map {name sfname afname} {
 #	The conversion result.
 
 proc ::doctools::toc::_format {name text} {
-    upvar ::doctools::toc::doctoc${name}::format format
+    upvar #0 ::doctools::toc::doctoc${name}::format format
     if {$format == ""} {
 	return -code error "$name: No format was specified"
     }
 
-    upvar ::doctools::toc::doctoc${name}::format_ip format_ip
-    upvar ::doctools::toc::doctoc${name}::chk_ip    chk_ip
-    upvar ::doctools::toc::doctoc${name}::ex_ok     ex_ok
-    upvar ::doctools::toc::doctoc${name}::expander  expander
-    upvar ::doctools::toc::doctoc${name}::passes    passes
-    upvar ::doctools::toc::doctoc${name}::msg       warnings
+    upvar #0 ::doctools::toc::doctoc${name}::format_ip format_ip
+    upvar #0 ::doctools::toc::doctoc${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::toc::doctoc${name}::ex_ok     ex_ok
+    upvar #0 ::doctools::toc::doctoc${name}::expander  expander
+    upvar #0 ::doctools::toc::doctoc${name}::passes    passes
+    upvar #0 ::doctools::toc::doctoc${name}::msg       warnings
 
     if {!$ex_ok}       {SetupExpander  $name}
     if {$chk_ip == ""} {SetupChecker   $name}
@@ -423,7 +430,7 @@ proc ::doctools::toc::_search {name path} {
     if {![file isdirectory $path]} {return -code error "$name search: path is not a directory"}
     if {![file readable    $path]} {return -code error "$name search: path cannot be read"}
 
-    upvar ::doctools::toc::doctoc${name}::paths paths
+    upvar #0 ::doctools::toc::doctoc${name}::paths paths
     set paths [linsert $paths 0 $path]
     return
 }
@@ -439,7 +446,7 @@ proc ::doctools::toc::_search {name path} {
 #	A list of warnings.
 
 proc ::doctools::toc::_warnings {name} {
-    upvar ::doctools::toc::doctoc${name}::msg msg
+    upvar #0 ::doctools::toc::doctoc${name}::msg msg
     return $msg
 }
 
@@ -455,7 +462,7 @@ proc ::doctools::toc::_warnings {name} {
 #	A list of parameter names
 
 proc ::doctools::toc::_parameters {name} {
-    upvar ::doctools::toc::doctoc${name}::param param
+    upvar #0 ::doctools::toc::doctoc${name}::param param
     return $param
 }
 
@@ -472,7 +479,7 @@ proc ::doctools::toc::_parameters {name} {
 #	None.
 
 proc ::doctools::toc::_setparam {name param value} {
-    upvar ::doctools::toc::doctoc${name}::format_ip format_ip
+    upvar #0 ::doctools::toc::doctoc${name}::format_ip format_ip
 
     if {$format_ip == {}} {
 	return -code error \
@@ -508,7 +515,7 @@ proc ::doctools::toc::LookupFormat {name format} {
 	return $format
     }
 
-    upvar ::doctools::toc::doctoc${name}::paths opaths
+    upvar #0 ::doctools::toc::doctoc${name}::paths opaths
     foreach path $opaths {
 	set f [file join $path toc.$format]
 	if {[file exists $f]} {
@@ -554,9 +561,10 @@ proc ::doctools::toc::SetupFormatter {name format} {
 
     $mpip invokehidden source [file join $here api_toc.tcl]
     #$mpip eval [list source [file join $here api_toc.tcl]]
-    interp alias $mpip dt_source   {} ::doctools::toc::Source $mpip [file dirname $format]
-    interp alias $mpip dt_package  {} ::doctools::Package $mpip
-    interp alias $mpip file        {} ::doctools::FileOp  $mpip
+    interp alias $mpip dt_source   {} ::doctools::toc::Source  $mpip [file dirname $format]
+    interp alias $mpip dt_read     {} ::doctools::toc::Read    $mpip [file dirname $format]
+    interp alias $mpip dt_package  {} ::doctools::toc::Package $mpip
+    interp alias $mpip file        {} ::doctools::toc::FileOp  $mpip
     interp alias $mpip puts_stderr {} ::puts stderr
     $mpip invokehidden source $format
     #$mpip eval [list source $format]
@@ -600,12 +608,12 @@ proc ::doctools::toc::SetupFormatter {name format} {
     # now invalid. It will be recreated during the
     # next call of 'format'.
 
-    upvar ::doctools::toc::doctoc${name}::formatfile formatfile
-    upvar ::doctools::toc::doctoc${name}::format_ip  format_ip
-    upvar ::doctools::toc::doctoc${name}::chk_ip     chk_ip
-    upvar ::doctools::toc::doctoc${name}::expander   expander
-    upvar ::doctools::toc::doctoc${name}::passes     xpasses
-    upvar ::doctools::toc::doctoc${name}::param      xparam
+    upvar #0 ::doctools::toc::doctoc${name}::formatfile formatfile
+    upvar #0 ::doctools::toc::doctoc${name}::format_ip  format_ip
+    upvar #0 ::doctools::toc::doctoc${name}::chk_ip     chk_ip
+    upvar #0 ::doctools::toc::doctoc${name}::expander   expander
+    upvar #0 ::doctools::toc::doctoc${name}::passes     xpasses
+    upvar #0 ::doctools::toc::doctoc${name}::param      xparam
 
     if {$chk_ip != {}}    {interp delete $chk_ip}
     if {$format_ip != {}} {interp delete $format_ip}
@@ -649,11 +657,11 @@ proc ::doctools::toc::SetupChecker {name} {
 
     variable here
 
-    upvar ::doctools::toc::doctoc${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::toc::doctoc${name}::chk_ip    chk_ip
     if {$chk_ip != ""} {return}
 
-    upvar ::doctools::toc::doctoc${name}::expander  expander
-    upvar ::doctools::toc::doctoc${name}::format_ip format_ip
+    upvar #0 ::doctools::toc::doctoc${name}::expander  expander
+    upvar #0 ::doctools::toc::doctoc${name}::format_ip format_ip
 
     set chk_ip [interp create] ; # interpreter hosting the formal format checker
 
@@ -698,10 +706,10 @@ proc ::doctools::toc::SetupChecker {name} {
 #	None.
 
 proc ::doctools::toc::SetupExpander {name} {
-    upvar ::doctools::toc::doctoc${name}::ex_ok    ex_ok
+    upvar #0 ::doctools::toc::doctoc${name}::ex_ok    ex_ok
     if {$ex_ok} {return}
 
-    upvar ::doctools::toc::doctoc${name}::expander expander
+    upvar #0 ::doctools::toc::doctoc${name}::expander expander
     ::textutil::expander $expander
     $expander evalcmd [list ::doctools::toc::Eval $name]
     $expander textcmd plain_text
@@ -721,7 +729,7 @@ proc ::doctools::toc::SetupExpander {name} {
 #	None.
 
 proc ::doctools::toc::SearchPaths {name} {
-    upvar ::doctools::toc::doctoc${name}::paths opaths
+    upvar #0 ::doctools::toc::doctoc${name}::paths opaths
     variable paths
 
     set p $opaths
@@ -756,7 +764,7 @@ proc ::doctools::toc::FmtError {name text} {
 #	None.
 
 proc ::doctools::toc::FmtWarning {name text} {
-    upvar ::doctools::toc::doctoc${name}::msg msg
+    upvar #0 ::doctools::toc::doctoc${name}::msg msg
     lappend msg $text
     return
 }
@@ -773,10 +781,11 @@ proc ::doctools::toc::FmtWarning {name text} {
 #	None.
 
 proc ::doctools::toc::Eval {name macro} {
-    upvar ::doctools::toc::doctoc${name}::chk_ip chk_ip
+    upvar #0 ::doctools::toc::doctoc${name}::chk_ip chk_ip
 
     # Handle the [include] command directly
     if {[string match include* $macro]} {
+	set macro [$chk_ip eval [list subst $macro]]
 	foreach {cmd filename} $macro break
 	return [ExpandInclude $name $filename]
     }
@@ -801,9 +810,9 @@ proc ::doctools::toc::ExpandInclude {name path} {
     # use the current working directory. Throw an error
     # if the file couldn't be found.
 
-    upvar ::doctools::toc::doctoc${name}::file file
+    upvar #0 ::doctools::toc::doctoc${name}::file file
 
-    set ipath [file join [file dirname $file] $path]
+    set ipath [file normalize [file join [file dirname $file] $path]]
     if {![file exists $ipath]} {
 	set ipath $path
 	if {![file exists $ipath]} {
@@ -815,9 +824,14 @@ proc ::doctools::toc::ExpandInclude {name path} {
     set    text [read $chan]
     close $chan
 
-    upvar ::doctools::toc::doctoc${name}::expander  expander
+    upvar #0 ::doctools::toc::doctoc${name}::expander  expander
 
-    return [$expander expand $text]
+    set saved $file
+    set file $ipath
+    set res [$expander expand $text]
+    set file $saved
+
+    return $res
 }
 
 # ::doctools::toc::GetUser --
@@ -846,7 +860,7 @@ proc ::doctools::toc::GetUser {name} {
 #	Format information
 
 proc ::doctools::toc::GetFormat {name} {
-    upvar ::doctools::toc::doctoc${name}::format format
+    upvar #0 ::doctools::toc::doctoc${name}::format format
     return $format
 }
 
@@ -864,7 +878,7 @@ proc ::doctools::toc::GetFormat {name} {
 #	Actual name of the file.
 
 proc ::doctools::toc::MapFile {name fname} {
-    upvar ::doctools::toc::doctoc${name}::map map
+    upvar #0 ::doctools::toc::doctoc${name}::map map
     if {[info exists map($fname)]} {
 	return $map($fname)
     }
@@ -888,6 +902,53 @@ proc ::doctools::toc::Source {ip path file} {
     return
 }
 
+proc ::doctools::toc::Read {ip path file} {
+    #puts stderr "$ip (read $path $file)"
+
+    return [read [set f [open [file join $path [file tail $file]]]]][close $f]
+}
+
+proc ::doctools::toc::FileOp {ip args} {
+    #puts stderr "$ip (file $args)"
+    # -- FUTURE -- disallow unsafe operations --
+
+    return [eval [linsert $args 0 file]]
+}
+
+proc ::doctools::toc::Package {ip pkg} {
+    #puts stderr "$ip package require $pkg"
+
+    set indexScript [Locate $pkg]
+
+    $ip expose source
+    $ip expose load
+    $ip eval		$indexScript
+    $ip hide   source
+    $ip hide   load
+    #$ip eval [list source [file join $path [file tail $file]]]
+    return
+}
+
+proc ::doctools::toc::Locate {p} {
+    # @mdgen NODEP: doctools::__undefined__
+    catch {package require doctools::__undefined__}
+
+    #puts stderr "auto_path = [join $::auto_path \n]"
+
+    # Check if requested package is in the list of loadable packages.
+    # Then get the highest possible version, and then the index script
+
+    if {[lsearch -exact [package names] $p] < 0} {
+	return -code error "Unknown package $p"
+    }
+
+    set v  [lindex [lsort -increasing [package versions $p]] end]
+
+    #puts stderr "Package $p = $v"
+
+    return [package ifneeded $p $v]
+}
+
 #------------------------------------
 # Module initialization
 
@@ -902,4 +963,4 @@ namespace eval ::doctools::toc {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools::toc 0.2.1
+package provide doctools::toc 1.1.3
