@@ -2,16 +2,23 @@
 #
 #	Implementation of doctools objects for Tcl.
 #
-# Copyright (c) 2003-2005 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2013 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: doctools.tcl,v 1.16 2005/09/30 05:36:39 andreas_kupries Exp $
+# RCS: @(#) $Id: doctools.tcl,v 1.45 2011/02/23 22:11:59 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require textutil::expander
 
+# @mdgen OWNER: api.tcl
+# @mdgen OWNER: checker.tcl
+# @mdgen OWNER: mpformats/*.tcl
+# @mdgen OWNER: mpformats/*.msg
+# @mdgen OWNER: mpformats/fmt.*
+# @mdgen OWNER: mpformats/man.macros
+
 namespace eval ::doctools {
     # Data storage in the doctools module
     # -------------------------------
@@ -123,6 +130,7 @@ proc ::doctools::help {} {
 	    * example       - example block\n\
 	    * example_begin - begin example\n\
 	    * example_end   - end of example\n\
+	    * category      - category declaration\n\
 	    * see_also      - cross reference declaration\n\
 	    * keywords      - keyword declaration\n\
 	    * nl            - paragraph break in list items\n\
@@ -178,6 +186,8 @@ proc ::doctools::new {name args} {
 	variable formatfile ""
 	variable deprecated 0
 	variable file       ""
+	variable mainfile   ""
+	variable ibase      ""
 	variable module     ""
 	variable copyright  ""
 	variable format_ip  ""
@@ -269,22 +279,24 @@ proc ::doctools::_cget {name option} {
 #	The value of one particular option if called with a single argument.
 
 proc ::doctools::_configure {name args} {
-    upvar ::doctools::doctools${name}::format_ip  format_ip
-    upvar ::doctools::doctools${name}::chk_ip     chk_ip
-    upvar ::doctools::doctools${name}::expander   expander
-    upvar ::doctools::doctools${name}::passes     passes
+    upvar #0 ::doctools::doctools${name}::format_ip  format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip     chk_ip
+    upvar #0 ::doctools::doctools${name}::expander   expander
+    upvar #0 ::doctools::doctools${name}::passes     passes
 
     if {[llength $args] == 0} {
 	# Retrieve the current configuration.
 
-	upvar ::doctools::doctools${name}::file       file
-	upvar ::doctools::doctools${name}::module     module
-	upvar ::doctools::doctools${name}::format     format
-	upvar ::doctools::doctools${name}::copyright  copyright
-	upvar ::doctools::doctools${name}::deprecated deprecated
+	upvar #0 ::doctools::doctools${name}::file       file
+	upvar #0 ::doctools::doctools${name}::ibase      ibase
+	upvar #0 ::doctools::doctools${name}::module     module
+	upvar #0 ::doctools::doctools${name}::format     format
+	upvar #0 ::doctools::doctools${name}::copyright  copyright
+	upvar #0 ::doctools::doctools${name}::deprecated deprecated
 
 	set     res [list]
 	lappend res -file       $file
+	lappend res -ibase      $ibase
 	lappend res -module     $module
 	lappend res -format     $format
 	lappend res -copyright  $copyright
@@ -296,29 +308,33 @@ proc ::doctools::_configure {name args} {
 
 	switch -exact -- [lindex $args 0] {
 	    -file {
-		upvar ::doctools::doctools${name}::file file
+		upvar #0 ::doctools::doctools${name}::file file
 		return $file
 	    }
+	    -ibase {
+		upvar #0 ::doctools::doctools${name}::ibase ibase
+		return $ibase
+	    }
 	    -module {
-		upvar ::doctools::doctools${name}::module module
+		upvar #0 ::doctools::doctools${name}::module module
 		return $module
 	    }
 	    -copyright {
-		upvar ::doctools::doctools${name}::copyright copyright
+		upvar #0 ::doctools::doctools${name}::copyright copyright
 		return $copyright
 	    }
 	    -format {
-		upvar ::doctools::doctools${name}::format format
+		upvar #0 ::doctools::doctools${name}::format format
 		return $format
 	    }
 	    -deprecated {
-		upvar ::doctools::doctools${name}::deprecated deprecated
+		upvar #0 ::doctools::doctools${name}::deprecated deprecated
 		return $deprecated
 	    }
 	    default {
 		return -code error \
 			"doctools::_configure: Unknown option \"[lindex $args 0]\", expected\
-			-copyright, -file, -module, -format, or -deprecated"
+			-copyright, -file, -ibase, -module, -format, or -deprecated"
 	    }
 	}
     } else {
@@ -331,22 +347,28 @@ proc ::doctools::_configure {name args} {
 	foreach {option value} $args {
 	    switch -exact -- $option {
 		-file {
-		    upvar ::doctools::doctools${name}::file file
-		    set file $value
+		    upvar #0 ::doctools::doctools${name}::file     file
+		    upvar #0 ::doctools::doctools${name}::mainfile mfile
+		    set file  $value
+		    set mfile $value
+		}
+		-ibase {
+		    upvar #0 ::doctools::doctools${name}::ibase    ibase
+		    set ibase $value
 		}
 		-module {
-		    upvar ::doctools::doctools${name}::module module
+		    upvar #0 ::doctools::doctools${name}::module module
 		    set module $value
 		}
 		-copyright {
-		    upvar ::doctools::doctools${name}::copyright copyright
+		    upvar #0 ::doctools::doctools${name}::copyright copyright
 		    set copyright $value
 		}
 		-format {
 		    if {[catch {
 			set fmtfile [LookupFormat $name $value]
 			SetupFormatter $name $fmtfile
-			upvar ::doctools::doctools${name}::format format
+			upvar #0 ::doctools::doctools${name}::format format
 			set format $value
 		    } msg]} {
 			return -code error "doctools::_configure: -format: $msg"
@@ -357,13 +379,13 @@ proc ::doctools::_configure {name args} {
 			return -code error \
 				"doctools::_configure: -deprecated expected a boolean, got \"$value\""
 		    }
-		    upvar ::doctools::doctools${name}::deprecated deprecated
+		    upvar #0 ::doctools::doctools${name}::deprecated deprecated
 		    set deprecated $value
 		}
 		default {
 		    return -code error \
 			    "doctools::_configure: Unknown option \"$option\", expected\
-			    -copyright, -file, -module, -format, or -deprecated"
+			    -copyright, -file, -ibase, -module, -format, or -deprecated"
 		}
 	    }
 	}
@@ -414,11 +436,34 @@ proc ::doctools::_destroy {name} {
 #	None.
 
 proc ::doctools::_map {name sfname afname} {
-    upvar ::doctools::doctools${name}::map map
+    upvar #0 ::doctools::doctools${name}::map map
     set map($sfname) $afname
     return
 }
 
+# ::doctools::_img --
+#
+
+#	Add a mapping from symbolic to the actual image filenames to
+#	the object. Two actual paths! The path the image is found at
+#	in the input, and the path for where image is to be placed in
+#	the output.
+#
+# Arguments:
+#	name	Name of the doctools object to use
+#	sfname	Symbolic filename to map
+#	afnameo	Actual filename, origin
+#	afnamed	Actual filename, destination
+#
+# Results:
+#	None.
+
+proc ::doctools::_img {name sfname afnameo afnamed} {
+    upvar #0 ::doctools::doctools${name}::imap imap
+    set imap($sfname) [list $afnameo $afnamed]
+    return
+}
+
 # ::doctools::_format --
 #
 #	Convert some text in doctools format
@@ -432,17 +477,17 @@ proc ::doctools::_map {name sfname afname} {
 #	The conversion result.
 
 proc ::doctools::_format {name text} {
-    upvar ::doctools::doctools${name}::format format
+    upvar #0 ::doctools::doctools${name}::format format
     if {$format == ""} {
 	return -code error "$name: No format was specified"
     }
 
-    upvar ::doctools::doctools${name}::format_ip format_ip
-    upvar ::doctools::doctools${name}::chk_ip    chk_ip
-    upvar ::doctools::doctools${name}::ex_ok     ex_ok
-    upvar ::doctools::doctools${name}::expander  expander
-    upvar ::doctools::doctools${name}::passes    passes
-    upvar ::doctools::doctools${name}::msg       warnings
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::doctools${name}::ex_ok     ex_ok
+    upvar #0 ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::passes    passes
+    upvar #0 ::doctools::doctools${name}::msg       warnings
 
     if {!$ex_ok}       {SetupExpander  $name}
     if {$chk_ip == ""} {SetupChecker   $name}
@@ -465,7 +510,7 @@ proc ::doctools::_format {name text} {
 	    catch {$format_ip eval fmt_shutdown}
 	    return -code error "Could not initialize pass $n of engine"
 	}
-	$chk_ip eval ck_initialize
+	$chk_ip eval ck_initialize $n
 
 	if {[catch {set result [$expander expand $text]} msg]} {
 	    catch {$format_ip eval fmt_shutdown}
@@ -510,7 +555,7 @@ proc ::doctools::_search {name path} {
     if {![file isdirectory $path]} {return -code error "$name search: path is not a directory"}
     if {![file readable    $path]} {return -code error "$name search: path cannot be read"}
 
-    upvar ::doctools::doctools${name}::paths paths
+    upvar #0 ::doctools::doctools${name}::paths paths
     set paths [linsert $paths 0 $path]
     return
 }
@@ -526,7 +571,7 @@ proc ::doctools::_search {name path} {
 #	A list of warnings.
 
 proc ::doctools::_warnings {name} {
-    upvar ::doctools::doctools${name}::msg msg
+    upvar #0 ::doctools::doctools${name}::msg msg
     return $msg
 }
 
@@ -542,7 +587,7 @@ proc ::doctools::_warnings {name} {
 #	A list of parameter names
 
 proc ::doctools::_parameters {name} {
-    upvar ::doctools::doctools${name}::param param
+    upvar #0 ::doctools::doctools${name}::param param
     return $param
 }
 
@@ -559,7 +604,7 @@ proc ::doctools::_parameters {name} {
 #	None.
 
 proc ::doctools::_setparam {name param value} {
-    upvar ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
 
     if {$format_ip == {}} {
 	return -code error \
@@ -595,7 +640,7 @@ proc ::doctools::LookupFormat {name format} {
 	return $format
     }
 
-    upvar ::doctools::doctools${name}::paths opaths
+    upvar #0 ::doctools::doctools${name}::paths opaths
     foreach path $opaths {
 	set f [file join $path fmt.$format]
 	if {[file exists $f]} {
@@ -643,6 +688,7 @@ proc ::doctools::SetupFormatter {name format} {
     $mpip invokehidden source [file join $here api.tcl]
     #$mpip eval [list source [file join $here api.tcl]]
     interp alias $mpip dt_source   {} ::doctools::Source  $mpip [file dirname $format]
+    interp alias $mpip dt_read     {} ::doctools::Read    $mpip [file dirname $format]
     interp alias $mpip dt_package  {} ::doctools::Package $mpip
     interp alias $mpip file        {} ::doctools::FileOp  $mpip
     interp alias $mpip puts_stderr {} ::puts stderr
@@ -688,12 +734,12 @@ proc ::doctools::SetupFormatter {name format} {
     # now invalid. It will be recreated during the
     # next call of 'format'.
 
-    upvar ::doctools::doctools${name}::formatfile formatfile
-    upvar ::doctools::doctools${name}::format_ip  format_ip
-    upvar ::doctools::doctools${name}::chk_ip     chk_ip
-    upvar ::doctools::doctools${name}::expander   expander
-    upvar ::doctools::doctools${name}::passes     xpasses
-    upvar ::doctools::doctools${name}::param      xparam
+    upvar #0 ::doctools::doctools${name}::formatfile formatfile
+    upvar #0 ::doctools::doctools${name}::format_ip  format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip     chk_ip
+    upvar #0 ::doctools::doctools${name}::expander   expander
+    upvar #0 ::doctools::doctools${name}::passes     xpasses
+    upvar #0 ::doctools::doctools${name}::param      xparam
 
     if {$chk_ip != {}}    {interp delete $chk_ip}
     if {$format_ip != {}} {interp delete $format_ip}
@@ -704,6 +750,7 @@ proc ::doctools::SetupFormatter {name format} {
     # Now link engine API into it.
 
     interp alias $mpip dt_file      {} ::doctools::GetFile      $name
+    interp alias $mpip dt_mainfile  {} ::doctools::GetMainFile  $name
     interp alias $mpip dt_fileid    {} ::doctools::GetFileId    $name
     interp alias $mpip dt_module    {} ::doctools::GetModule    $name
     interp alias $mpip dt_copyright {} ::doctools::GetCopyright $name
@@ -711,6 +758,9 @@ proc ::doctools::SetupFormatter {name format} {
     interp alias $mpip dt_user      {} ::doctools::GetUser      $name
     interp alias $mpip dt_lnesting  {} ::doctools::ListLevel    $name
     interp alias $mpip dt_fmap      {} ::doctools::MapFile      $name
+    interp alias $mpip dt_imgsrc    {} ::doctools::ImgSrc       $name
+    interp alias $mpip dt_imgdst    {} ::doctools::ImgDst       $name
+    interp alias $mpip dt_imgdata   {} ::doctools::ImgData      $name
     interp alias $mpip file         {} ::doctools::FileCmd
 
     foreach cmd {cappend cget cis cname cpop cpush ctopandclear cset lb rb} {
@@ -743,11 +793,11 @@ proc ::doctools::SetupChecker {name} {
 
     variable here
 
-    upvar ::doctools::doctools${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip    chk_ip
     if {$chk_ip != ""} {return}
 
-    upvar ::doctools::doctools${name}::expander  expander
-    upvar ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
 
     set chk_ip [interp create] ; # interpreter hosting the formal format checker
 
@@ -758,6 +808,8 @@ proc ::doctools::SetupChecker {name} {
 	dt_deprecated Deprecated
 	dt_error      FmtError
 	dt_warning    FmtWarning
+	dt_where      Where
+	dt_file       GetFile
     } {
 	interp alias $chk_ip $cmd {} ::doctools::$ckcmd $name
     }
@@ -780,7 +832,7 @@ proc ::doctools::SetupChecker {name} {
 	keywords nl arg cmd opt comment sectref syscmd method option
 	widget fun type package class var file uri usage term const
 	arg_def cmd_def opt_def tkoption_def emph strong plain_text
-	namespace subsection
+	namespace subsection category image
     } {
 	interp alias $chk_ip fmt_$cmd $format_ip fmt_$cmd
     }
@@ -798,10 +850,10 @@ proc ::doctools::SetupChecker {name} {
 #	None.
 
 proc ::doctools::SetupExpander {name} {
-    upvar ::doctools::doctools${name}::ex_ok    ex_ok
+    upvar #0 ::doctools::doctools${name}::ex_ok    ex_ok
     if {$ex_ok} {return}
 
-    upvar ::doctools::doctools${name}::expander expander
+    upvar #0 ::doctools::doctools${name}::expander expander
     ::textutil::expander $expander
     $expander evalcmd [list ::doctools::Eval $name]
     $expander textcmd plain_text
@@ -821,7 +873,7 @@ proc ::doctools::SetupExpander {name} {
 #	None.
 
 proc ::doctools::SearchPaths {name} {
-    upvar ::doctools::doctools${name}::paths opaths
+    upvar #0 ::doctools::doctools${name}::paths opaths
     variable paths
 
     set p $opaths
@@ -841,7 +893,7 @@ proc ::doctools::SearchPaths {name} {
 #	None.
 
 proc ::doctools::Deprecated {name} {
-    upvar ::doctools::doctools${name}::deprecated deprecated
+    upvar #0 ::doctools::doctools${name}::deprecated deprecated
     return $deprecated
 }
 
@@ -872,11 +924,26 @@ proc ::doctools::FmtError {name text} {
 #	None.
 
 proc ::doctools::FmtWarning {name text} {
-    upvar ::doctools::doctools${name}::msg msg
+    upvar #0 ::doctools::doctools${name}::msg msg
     lappend msg $text
     return
 }
 
+# ::doctools::Where --
+#
+#	API for checker. Called when the current location is needed
+#
+# Arguments:
+#	name	Name of the doctools object
+#
+# Results:
+#	List containing offset, line, column
+
+proc ::doctools::Where {name} {
+    upvar #0 ::doctools::doctools${name}::expander expander
+    return [$expander where]
+}
+
 # ::doctools::Eval --
 #
 #	API for expander. Routes the macro invocations
@@ -889,12 +956,13 @@ proc ::doctools::FmtWarning {name text} {
 #	None.
 
 proc ::doctools::Eval {name macro} {
-    upvar ::doctools::doctools${name}::chk_ip chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip chk_ip
 
     #puts stderr "\t\t$name [lindex [split $macro] 0]"
 
     # Handle the [include] command directly
     if {[string match include* $macro]} {
+	set macro [$chk_ip eval [list subst $macro]]
 	foreach {cmd filename} $macro break
 	return [ExpandInclude $name $filename]
     }
@@ -923,9 +991,16 @@ proc ::doctools::Eval {name macro} {
 #	None.
 
 proc ::doctools::ExpandInclude {name path} {
-    upvar ::doctools::doctools${name}::file file
+    upvar #0 ::doctools::doctools${name}::file file
+    upvar #0 ::doctools::doctools${name}::ibase ibase
 
-    set ipath [file join [file dirname $file] $path]
+    set savedi $ibase
+    set savedf $file
+
+    set base $ibase
+    if {$base eq {}} { set base $file }
+
+    set ipath [file normalize [file join [file dirname $base] $path]]
     if {![file exists $ipath]} {
 	set ipath $path
 	if {![file exists $ipath]} {
@@ -937,9 +1012,15 @@ proc ::doctools::ExpandInclude {name path} {
     set    text [read $chan]
     close $chan
 
-    upvar ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::expander  expander
 
-    return [$expander expand $text]
+    set ibase $ipath
+    set res [$expander expand $text]
+
+    set ibase $savedi
+    set file  $savedf
+
+    return $res
 }
 
 # ::doctools::GetUser --
@@ -971,12 +1052,22 @@ proc ::doctools::GetFile {name} {
 
     #puts stderr "GetFile $name"
 
-    upvar ::doctools::doctools${name}::file file
+    upvar #0 ::doctools::doctools${name}::file file
 
     #puts stderr "ok $file"
     return $file
 }
 
+proc ::doctools::GetMainFile {name} {
+
+    #puts stderr "GetMainFile $name"
+
+    upvar #0 ::doctools::doctools${name}::mainfile mfile
+
+    #puts stderr "ok $mfile"
+    return $mfile
+}
+
 # ::doctools::GetFileId --
 #
 #	API for formatter. Returns file information (truncated to stem of filename)
@@ -1020,7 +1111,7 @@ proc ::doctools::FileCmd {cmd args} {
 #	Module information
 
 proc ::doctools::GetModule {name} {
-    upvar ::doctools::doctools${name}::module module
+    upvar #0 ::doctools::doctools${name}::module module
     return   $module
 }
 
@@ -1035,7 +1126,7 @@ proc ::doctools::GetModule {name} {
 #	Copyright information
 
 proc ::doctools::GetCopyright {name} {
-    upvar ::doctools::doctools${name}::copyright copyright
+    upvar #0 ::doctools::doctools${name}::copyright copyright
     return   $copyright
 }
 
@@ -1050,7 +1141,7 @@ proc ::doctools::GetCopyright {name} {
 #	Format information
 
 proc ::doctools::GetFormat {name} {
-    upvar ::doctools::doctools${name}::format format
+    upvar #0 ::doctools::doctools${name}::format format
     return $format
 }
 
@@ -1065,7 +1156,7 @@ proc ::doctools::GetFormat {name} {
 #	Boolean flag.
 
 proc ::doctools::ListLevel {name} {
-    upvar ::doctools::doctools${name}::chk_ip chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip chk_ip
     return [$chk_ip eval LNest]
 }
 
@@ -1083,7 +1174,7 @@ proc ::doctools::ListLevel {name} {
 #	Actual name of the file.
 
 proc ::doctools::MapFile {name fname} {
-    upvar ::doctools::doctools${name}::map map
+    upvar #0 ::doctools::doctools${name}::map map
 
     #parray map
 
@@ -1093,6 +1184,79 @@ proc ::doctools::MapFile {name fname} {
     return $fname
 }
 
+# ::doctools::Img{Src,Dst} --
+#
+#	API for formatter. Maps symbolic to actual image in a doctools
+#	item. Returns nothing if no mapping is found.
+#
+# Arguments:
+#	name		Name of the doctools object to query.
+#	iname		Symbolic name of the image file.
+#	extensions	List of acceptable file extensions.
+#
+# Results:
+#	Actual name of the file.
+
+proc ::doctools::ImgData {name iname extensions} {
+
+    # The system searches for the image relative to the current input
+    # file, and the current main file
+
+    upvar #0 ::doctools::doctools${name}::imap imap
+
+    #parray imap
+
+    foreach e $extensions {
+	if {[info exists imap($iname.$e)]} {
+	    foreach {origin dest} $imap($iname.$e) break
+
+	    set f   [open $origin r]
+	    set img [read $f]
+	    close   $f
+
+	    return $img
+	}
+    }
+    return {}
+}
+
+proc ::doctools::ImgSrc {name iname extensions} {
+
+    # The system searches for the image relative to the current input
+    # file, and the current main file
+
+    upvar #0 ::doctools::doctools${name}::imap imap
+
+    #parray imap
+
+    foreach e $extensions {
+	if {[info exists imap($iname.$e)]} {
+	    foreach {origin dest} $imap($iname.$e) break
+	    return $origin
+	}
+    }
+    return {}
+}
+
+proc ::doctools::ImgDst {name iname extensions} {
+    # The system searches for the image relative to the current input
+    # file, and the current main file
+
+    upvar #0 ::doctools::doctools${name}::imap imap
+
+    #parray imap
+
+    foreach e $extensions {
+	if {[info exists imap($iname.$e)]} {
+	    foreach {origin dest} $imap($iname.$e) break
+	    file mkdir [file dirname $dest]
+	    file copy -force $origin $dest
+	    return $dest
+	}
+    }
+    return {}
+}
+
 # ::doctools::Source --
 #
 #	API for formatter. Used by engine to ask for
@@ -1112,8 +1276,14 @@ proc ::doctools::Source {ip path file} {
     return
 }
 
+proc ::doctools::Read {ip path file} {
+    #puts stderr "$ip (read $path $file)"
+
+    return [read [set f [open [file join $path [file tail $file]]]]][close $f]
+}
 
 proc ::doctools::Locate {p} {
+    # @mdgen NODEP: doctools::__undefined__
     catch {package require doctools::__undefined__}
 
     #puts stderr "auto_path = [join $::auto_path \n]"
@@ -1139,7 +1309,6 @@ proc ::doctools::FileOp {ip args} {
     return [eval [linsert $args 0 file]]
 }
 
-
 proc ::doctools::Package {ip pkg} {
     #puts stderr "$ip package require $pkg"
 
@@ -1168,4 +1337,4 @@ namespace eval ::doctools {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools 1.2
+package provide doctools 1.4.14
