@@ -13,7 +13,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: ftp.tcl,v 1.42 2006/09/22 23:08:04 andreas_kupries Exp $
+# RCS: @(#) $Id: ftp.tcl,v 1.50 2011/08/09 20:57:01 andreas_kupries Exp $
 #
 #   core ftp support: 	ftp::Open <server> <user> <passwd> <?options?>
 #			ftp::Close <s>
@@ -58,9 +58,9 @@ namespace eval ::ftp {
 	    FileSize ModTime Delete Rename Put Append Get Reget \
 	    Newer Quote MkDir RmDir
 
-    set serial 0
-    set VERBOSE 0
-    set DEBUG 0
+    variable serial  0
+    variable VERBOSE 0
+    variable DEBUG   0
 }
 
 #############################################################################
@@ -99,10 +99,11 @@ proc ::ftp::DisplayMsg {s msg {state ""}} {
     #              variable VERBOSE is still relevant as it controls
     #              whether this procedure is called or not.
 
+    global errorInfo
     switch -exact -- $state {
         data    {log::log debug "$state | $msg"}
         control {log::log debug "$state | $msg"}
-        error   {log::log error "$state | E: $msg"}
+        error   {log::log error "$state | E: $msg:\n$errorInfo"}
         default {log::log debug "$state | $msg"}
     }
     return
@@ -260,24 +261,45 @@ proc ::ftp::StateHandler {s {sock ""}} {
 		    DisplayMsg $s $errmsg control
 		}
 	    } else {
-		set buffer $bufline
-			
 		# multi-line format detected ("-"), get all the lines
 		# until the real return code
 
+		set buffer $bufline
+			
 		while { [string equal $multi_line "-"] } {
 		    set number [gets $sock bufline]	
 		    if { $number > 0 } {
 			append buffer \n "$bufline"
 			regexp -- "(^\[0-9\]+)( |-)?(.*)$" $bufline all rc multi_line
+			# multi_line is not set if the bufline does not match the regexp,
+			# I.e. this keeps the '-' which started this around until the
+			# closing line does match and sets it to space.
+		    } elseif {$number == -1 && [eof $sock]} {
+			# The reply indicated a multi-line reply, but the
+			# socket was closed and there were no more lines.
+			# In that case, keep the current return values.
+
+			# This means the server isn't speaking strict rfc959.
+			# see section on multi-line replies
+			break
 		    }
 		}
+
+		# Export the accumulated response. [Bug 1191607].
+		set msgtext $buffer
 	    }
         } elseif { [eof $ftp(CtrlSock)] } {
-            # remote server has closed control connection
-            # kill control socket, unset State to disable all following command
+            # remote server has closed control connection. kill
+            # control socket, unset State to disable all following
+            # commands. Killing the socket is done before
+            # 'WaitComplete' to prevent it from recursively entering
+            # this code, overflowing the stack (socket still existing,
+            # still readable, still eof). [SF Tcllib Bug 15822535].
             
             set rc 421
+            catch {close $ftp(CtrlSock)}
+            catch {unset  ftp(CtrlSock)}
+            catch {unset  ftp(state.data)}
             if { $VERBOSE } {
                 DisplayMsg $s "C: 421 Service not available, closing control connection." control
             }
@@ -288,7 +310,6 @@ proc ::ftp::StateHandler {s {sock ""}} {
             WaitComplete $s 0
 	    Command $ftp(Command) terminated
             catch {unset ftp(State)}
-            catch {close $ftp(CtrlSock); unset ftp(CtrlSock)}
             return
         } else {
 	    # Fix SF bug #466746: Incomplete line, do nothing.
@@ -507,7 +528,7 @@ proc ::ftp::StateHandler {s {sock ""}} {
                 }
                 default {  
                     if { [string equal $ftp(Mode) "passive"] } {
-                        unset ftp(state.data)
+                        catch {unset ftp(state.data)}
                     }    
                     set errmsg "Error getting directory listing!"
                     set complete_with 0
@@ -753,7 +774,7 @@ proc ::ftp::StateHandler {s {sock ""}} {
                 default {
                     if { [string equal $ftp(Mode) "passive"] } {
                         # close already opened DataConnection
-                        unset ftp(state.data)
+                        catch {unset ftp(state.data)}
                     }  
                     set errmsg "Error opening connection!"
                     set complete_with 0
@@ -826,7 +847,7 @@ proc ::ftp::StateHandler {s {sock ""}} {
                 default {
                     if { [string equal $ftp(Mode) "passive"] } {
                         # close already opened DataConnection
-                        unset ftp(state.data)
+                        catch {unset ftp(state.data)}
                     }  
                     set errmsg "Error opening connection!"
                     set complete_with 0
@@ -912,7 +933,7 @@ proc ::ftp::StateHandler {s {sock ""}} {
                 default {
                     if { [string equal $ftp(Mode) "passive"] } {
                         # close already opened DataConnection
-                        unset ftp(state.data)
+                        catch {unset ftp(state.data)}
                     }  
                     set errmsg "Error retrieving file \"$ftp(RemoteFilename)\"!"
                     set complete_with 0
@@ -982,7 +1003,7 @@ proc ::ftp::StateHandler {s {sock ""}} {
                 default {
                     if { [string equal $ftp(Mode) "passive"] } {
                         # close already opened DataConnection
-                        unset ftp(state.data)
+                        catch {unset ftp(state.data)}
                     }  
                     set errmsg "Error retrieving file \"$ftp(RemoteFilename)\"!"
                     set complete_with 0
@@ -2019,7 +2040,15 @@ proc ::ftp::Get {s args} {
 		set returnData $ftp(GetData)
 	    }
 	}
-	catch {unset ftp(get:channel)}
+	# catch {unset ftp(get:channel)}
+	# SF Bug 1708350. DISABLED. In async mode (Open -command) the
+	# unset here causes HandleData to blow up, see marker <@>. In
+	# essence in async mode HandleData can be entered multiple
+	# times, and unsetting get:channel here causes it to think
+	# that the data goes into a local file, not a channel, but the
+	# state does not contain local file information, so an error
+	# is thrown. Removing the catch here seems to fix it without
+	# adverse effects elsewhere. Maybe. We hope.
         return 1
     } else {
         if {$ftp(inline)} {
@@ -2496,7 +2525,7 @@ proc ::ftp::CopyNext {s bytes {error {}}} {
 	    catch {close $ftp(DestCI)}
 	}
         catch {close $ftp(SourceCI)}
-        unset ftp(state.data)
+        catch {unset ftp(state.data)}
         DisplayMsg $s $error error
 
     } elseif { ([eof $ftp(SourceCI)] || ($blocksize <= 0)) } {
@@ -2507,7 +2536,7 @@ proc ::ftp::CopyNext {s bytes {error {}}} {
 	    close $ftp(DestCI)
 	}
         close $ftp(SourceCI)
-        unset ftp(state.data)
+        catch {unset ftp(state.data)}
         if { $VERBOSE } {
             DisplayMsg $s "D: Port closed" data
         }
@@ -2544,6 +2573,7 @@ proc ::ftp::HandleData {s sock} {
 	# A channel was specified by the caller. Use that instead of a
 	# file.
 
+	# SF Bug 1708350 <@>
 	if {[info exists ftp(get:channel)]} {
 	    set ftp(DestCI) $ftp(get:channel)
 	    set rc 0
@@ -2688,7 +2718,7 @@ proc ::ftp::HandleOutput {s sock} {
                 [expr {$ftp(Total) + $ftp(Blocksize)}]]
         if {[catch {puts -nonewline $sock "$substr"} result]} {
             close $sock
-            unset ftp(state.data)
+            catch {unset ftp(state.data)}
             if { $VERBOSE } {
                 DisplayMsg $s "D: Port closed" data
             }
@@ -2842,6 +2872,12 @@ proc ::ftp::OpenActiveConn {s } {
         return 0
     }
 
+    # prepare local ip address for PORT command (convert pointed format
+    # to comma format)
+
+    set ftp(LocalAddr) [lindex [fconfigure $ftp(CtrlSock) -sockname] 0]
+    set ftp(LocalAddr) [string map {. ,} $ftp(LocalAddr)]
+
     # get a new local port address for data transfer and convert it to a format
     # which is useable by the PORT command
 
@@ -2927,17 +2963,10 @@ proc ::ftp::OpenControlConn {s {block 1}} {
     fconfigure $ftp(CtrlSock) -buffering line -blocking $block -translation {auto crlf}
     fileevent $ftp(CtrlSock) readable [list [namespace current]::StateHandler $s $ftp(CtrlSock)]
 	
-    # prepare local ip address for PORT command (convert pointed format
-    # to comma format)
-
-    set ftp(LocalAddr) [lindex [fconfigure $ftp(CtrlSock) -sockname] 0]
-    set ftp(LocalAddr) [string map {. ,} $ftp(LocalAddr)]
-
     # report ready message
 
-    set peer [fconfigure $ftp(CtrlSock) -peername]
     if { $VERBOSE } {
-        DisplayMsg $s "C: Connection from [lindex $peer 0]:[lindex $peer 2]" control
+        DisplayMsg $s "C: Connection to $ftp(RemoteHost):$ftp(Port)" control
     }
 	
     return 1
@@ -2970,9 +2999,11 @@ if { [string equal [uplevel "#0" {info commands tkcon}] "tkcon"] } {
 
     # new ftp::List proc makes the output more readable
     proc ::ftp::__ftp_ls {args} {
-        foreach i [eval ::ftp::List_org $args] {
+	set rc [eval [linsert $args 0 ::ftp::List_org]]
+        foreach i $rc {
             puts $i
         }
+	return $rc
     }
 
     # rename the original ftp::List procedure
@@ -2988,4 +3019,4 @@ if { [string equal [uplevel "#0" {info commands tkcon}] "tkcon"] } {
 # ==================================================================
 # At last, everything is fine, we can provide the package.
 
-package provide ftp [lindex {Revision: 2.4.4} 1]
+package provide ftp [lindex {Revision: 2.4.11} 1]
