@@ -1,4 +1,4 @@
-# autoproxy.tcl - Copyright (C) 2002 Pat Thoyts <patthoyts@users.sf.net>
+# autoproxy.tcl - Copyright (C) 2002-2008 Pat Thoyts <patthoyts@users.sf.net>
 #
 # On Unix the standard for identifying the local HTTP proxy server
 # seems to be to use the environment variable http_proxy or ftp_proxy and
@@ -7,7 +7,7 @@
 # On Windows we can retrieve the Internet Settings values from the registry
 # to obtain pretty much the same information.
 #
-# With this information we can setup a suitable filter procedure for the 
+# With this information we can setup a suitable filter procedure for the
 # Tcl http package and arrange for automatic use of the proxy.
 #
 # Example:
@@ -20,15 +20,15 @@
 #   package require tls
 #   http::register https 443 ::autoproxy::tls_socket
 #
-# @(#)$Id: autoproxy.tcl,v 1.9 2007/03/12 22:53:25 patthoyts Exp $
+# @(#)$Id: autoproxy.tcl,v 1.15 2012/05/28 20:30:32 andreas_kupries Exp $
 
 package require http;                   # tcl
 package require uri;                    # tcllib
 package require base64;                 # tcllib
 
 namespace eval ::autoproxy {
-    variable rcsid {$Id: autoproxy.tcl,v 1.9 2007/03/12 22:53:25 patthoyts Exp $}
-    variable version 1.4
+    variable rcsid {$Id: autoproxy.tcl,v 1.15 2012/05/28 20:30:32 andreas_kupries Exp $}
+    variable version 1.5.3
     variable options
 
     if {! [info exists options]} {
@@ -36,11 +36,14 @@ namespace eval ::autoproxy {
             proxy_host ""
             proxy_port 80
             no_proxy   {}
-            basic      {} 
+            basic      {}
             authProc   {}
         }
     }
 
+    variable uid
+    if {![info exists uid]} { set uid 0 }
+
     variable winregkey
     set winregkey [join {
         HKEY_CURRENT_USER
@@ -55,7 +58,7 @@ namespace eval ::autoproxy {
 #
 proc ::autoproxy::cget {option} {
     variable options
-    switch -glob -- $option] {
+    switch -glob -- $option {
         -host -
         -proxy_h* { set options(proxy_host) }
         -port -
@@ -94,13 +97,13 @@ proc ::autoproxy::configure {args} {
 
     while {[string match "-*" [set option [lindex $args 0]]]} {
         switch -glob -- $option {
-            -host - 
+            -host -
             -proxy_h* { set options(proxy_host) [Pop args 1]}
-            -port - 
+            -port -
             -proxy_p* { set options(proxy_port) [Pop args 1]}
             -no*      { set options(no_proxy) [Pop args 1] }
             -basic    { Pop args; configure:basic $args ; break }
-            -authProc { set options(authProc) [Pop args] }
+            -authProc { set options(authProc) [Pop args 1] }
             --        { Pop args; break }
             default {
                 set opts [join [lsort [array names options]] ", -"]
@@ -131,7 +134,7 @@ proc ::autoproxy::init {{httpproxy {}} {no_proxy {}}} {
 
     # Look for standard environment variables.
     if {[string length $httpproxy] > 0} {
-        
+
         # nothing to do
 
     } elseif {[info exists env(http_proxy)]} {
@@ -141,6 +144,7 @@ proc ::autoproxy::init {{httpproxy {}} {no_proxy {}}} {
         }
     } else {
         if {$tcl_platform(platform) == "windows"} {
+            #checker -scope block exclude nonPortCmd
             package require registry 1.0
             array set reg {ProxyEnable 0 ProxyServer "" ProxyOverride {}}
             catch {
@@ -153,7 +157,7 @@ proc ::autoproxy::init {{httpproxy {}} {no_proxy {}}} {
                         set v [registry get $winregkey "ProxyEnable"]
                         binary scan $v i reg(ProxyEnable)
                     }
-                    default { 
+                    default {
                         return -code error "unexpected type found for\
                                ProxyEnable registry item"
                     }
@@ -170,14 +174,14 @@ proc ::autoproxy::init {{httpproxy {}} {no_proxy {}}} {
             }
         }
     }
-    
+
     # If we found something ...
     if {[string length $httpproxy] > 0} {
         # The http_proxy is supposed to be a URL - lets make sure.
         if {![regexp {\w://.*} $httpproxy]} {
             set httpproxy "http://$httpproxy"
         }
-        
+
         # decompose the string.
         array set proxy [uri::split $httpproxy]
 
@@ -209,16 +213,17 @@ proc ::autoproxy::init {{httpproxy {}} {no_proxy {}}} {
     return $httpproxy
 }
 
-# autoproxy::GetWin32Proxy -- 
+# autoproxy::GetWin32Proxy --
 #
 #	Parse the Windows Internet Settings registry key and return the
-#	protocol proxy requested. If the same proxy is in use for all 
+#	protocol proxy requested. If the same proxy is in use for all
 #	protocols, then that will be returned. Otherwise the string is
 #	parsed. Example:
 #	 ftp=proxy:80;http=proxy:80;https=proxy:80
 #
 proc ::autoproxy::GetWin32Proxy {protocol} {
     variable winregkey
+    #checker exclude nonPortCmd
     set proxies [split [registry get $winregkey "ProxyServer"] ";"]
     foreach proxy $proxies {
         if {[string first = $proxy] == -1} {
@@ -247,8 +252,8 @@ proc ::autoproxy::Pop {varname {nth 0}} {
 # Description
 #   An example user authentication procedure.
 # Returns:
-#   A two element list consisting of the users authentication id and 
-#   password. 
+#   A two element list consisting of the users authentication id and
+#   password.
 proc ::autoproxy::defAuthProc {{user {}} {passwd {}} {realm {}}} {
     if {[string length $realm] > 0} {
         set title "Realm: $realm"
@@ -263,9 +268,10 @@ proc ::autoproxy::defAuthProc {{user {}} {passwd {}} {realm {}}} {
     #         -title $title -logintext $user -passwdtext $passwd]
     #
     # if you just have Tk and no BWidgets --
-    
+
     set dlg [toplevel .autoproxy_defAuthProc -class Dialog]
-    wm title $dlg "$realm"
+    wm title $dlg $title
+    wm withdraw $dlg
     label $dlg.ll -text Login -underline 0 -anchor w
     entry $dlg.le -textvariable [namespace current]::${dlg}:l
     label $dlg.pl -text Password -underline 0 -anchor w
@@ -347,13 +353,13 @@ proc ::autoproxy::filter {host} {
     if {$options(proxy_host) == {}} {
         return {}
     }
-    
+
     foreach domain $options(no_proxy) {
         if {[string match $domain $host]} {
             return {}
         }
     }
-    
+
     # Add authorisation header to the request (by Anders Ramdahl)
     catch {
         upvar state State
@@ -364,44 +370,26 @@ proc ::autoproxy::filter {host} {
     return [list $options(proxy_host) $options(proxy_port)]
 }
 
+# -------------------------------------------------------------------------
 # autoproxy::tls_connect --
 #
 #	Create a connection to a remote machine through a proxy
-#	if necessary. This is used by the tls_socket comment for 
+#	if necessary. This is used by the tls_socket command for
 #	use with the http package but can also be used more generally
 #	provided your proxy will permit CONNECT attempts to ports
 #	other than port 443 (many will not).
+#	This command defers to 'tunnel_connect' to link to the target
+#	host and then upgrades the link to SSL/TLS
 #
-proc ::autoproxy::tls_connect {host port {useragent {}}} {
+proc ::autoproxy::tls_connect {args} {
     variable options
     if {[string length $options(proxy_host)] > 0} {
-        set s [::socket $options(proxy_host) $options(proxy_port)]
-        fconfigure $s -blocking 1 -buffering line -translation crlf
-        puts $s "CONNECT $host:$port HTTP/1.1"
-        puts $s "Host: $host"
-        if {[string length $useragent] > 0} {
-            puts $s "User-Agent: $useragent"
-        }
-        puts $s "Proxy-Connection: keep-alive"
-        puts $s "Connection: keep-alive"
-        if {[string length $options(basic)] > 0} {
-            puts $s [join $options(basic) ": "]
-        }
-        puts $s ""
-
-        set block ""
-        while {[gets $s r] > 0} {
-            lappend block $r
-        }
-        set result [lindex $block 0]
-        set code [lindex [split $result { }] 1]
-
-        if {$code >= 200 && $code < 300} {
-            fconfigure $s -blocking 1 -buffering none -translation binary
-            tls::import $s
+        set s [eval [linsert $args 0 tunnel_connect]]
+        fconfigure $s -blocking 1 -buffering none -translation binary
+        if {[string equal "-async" [lindex $args end-2]]} {
+            eval [linsert [lrange $args 0 end-3] 0 ::tls::import $s]
         } else {
-            close $s
-            return -code error $result
+            eval [linsert [lrange $args 0 end-2] 0 ::tls::import $s]
         }
     } else {
         set s [eval [linsert $args 0 ::tls::socket]]
@@ -409,9 +397,114 @@ proc ::autoproxy::tls_connect {host port {useragent {}}} {
     return $s
 }
 
+# autoproxy::tunnel_connect --
+#
+#	Create a connection to a remote machine through a proxy
+#	if necessary. This is used by the tls_socket command for
+#	use with the http package but can also be used more generally
+#	provided your proxy will permit CONNECT attempts to ports
+#	other than port 443 (many will not).
+#	Note: this command just opens the socket through the proxy to
+#	the target machine -- no SSL/TLS negotiation is done yet.
+#
+proc ::autoproxy::tunnel_connect {args} {
+    variable options
+    variable uid
+    set code ok
+
+    # args = ... host port
+    # and the host/port is the actual endpoint we want to talk to,
+    # regardless of any proxying. See our caller tls_connect for
+    # ensuring this by peeking into the http package internals.
+
+    # To handle proxying properly we have to run through 'filter'
+    # (again), to ensure that proxy exceptions are correctly taken
+    # into account.
+
+    set proxy [filter [lindex $args end-1]]
+
+    if {[llength $proxy]} {
+        foreach {proxy_host proxy_port} $proxy break
+
+        set token [namespace current]::[incr uid]
+        upvar #0 $token state
+        set state(endpoint) [lrange $args end-1 end]
+        set state(state) connect
+        set state(data) ""
+        set state(useragent) [http::config -useragent]
+        set state(sock) [::socket $proxy_host $proxy_port]
+        fileevent $state(sock) writable [namespace code [list tunnel_write $token]]
+        vwait [set token](state)
+
+        if {[string length $state(error)] > 0} {
+            set result $state(error)
+            close $state(sock)
+            unset state
+            set code error
+        } elseif {[info exists state(code)] &&
+                  (($state(code) >= 300) ||
+                   ($state(code) < 200))} {
+            set result [lindex $state(headers) 0]
+            regexp {HTTP/\d.\d\s+\d+\s+(.*)} $result -> result
+            close $state(sock)
+            set code error
+        } else {
+            set result $state(sock)
+        }
+        unset state
+    } else {
+        set result [eval [linsert $args 0 ::socket]]
+    }
+    return -code $code $result
+}
+
+proc ::autoproxy::tunnel_write {token} {
+    upvar #0 $token state
+    variable options
+    fileevent $state(sock) writable {}
+    if {[catch {set state(error) [fconfigure $state(sock) -error]} err]} {
+        set state(error) $err
+    }
+    if {[string length $state(error)] > 0} {
+        set state(state) error
+        return
+    }
+    fconfigure $state(sock) -blocking 0 -buffering line -translation crlf
+    foreach {host port} $state(endpoint) break
+    puts $state(sock) "CONNECT $host:$port HTTP/1.1"
+    puts $state(sock) "Host: $host"
+    if {[string length $state(useragent)] > 0} {
+        puts $state(sock) "User-Agent: $state(useragent)"
+    }
+    puts $state(sock) "Proxy-Connection: keep-alive"
+    puts $state(sock) "Connection: keep-alive"
+    if {[string length $options(basic)] > 0} {
+        puts $state(sock) [join $options(basic) ": "]
+    }
+    puts $state(sock) ""
+
+    fileevent $state(sock) readable [namespace code [list tunnel_read $token]]
+    return
+}
+
+proc ::autoproxy::tunnel_read {token} {
+    upvar #0 $token state
+    set len [gets $state(sock) line]
+    if {[eof $state(sock)]} {
+        fileevent $state(sock) readable {}
+        set state(state) eof
+    } elseif {$len == 0} {
+        set state(code) [lindex [split [lindex $state(headers) 0] { }] 1]
+        fileevent $state(sock) readable {}
+        set state(state) ok
+    } else {
+        lappend state(headers) $line
+    }
+}
+
 # autoproxy::tls_socket --
 #
-#	This can be used to handle TLS connections indenpendantly of
+#	This can be used to handle TLS connections independently of
 #	proxy presence. It can only be used with the Tcl http package
 #	and to use it you must do:
 #	   http::register https 443 ::autoproxy::tls_socket
@@ -421,10 +514,14 @@ proc ::autoproxy::tls_connect {host port {useragent {}}} {
 proc ::autoproxy::tls_socket {args} {
     variable options
 
-    # Look into the http package for the actual target. The function
-    # has unfortunately not passed these as parameters.
-    upvar host host port port
-    set s [tls_connect $host $port [http::config -useragent]]
+    # Look into the http package for the actual target. If a proxy is in use then
+    # The function appends the proxy host and port and not the target.
+
+    upvar host uhost port uport
+    set args [lrange $args 0 end-2]
+    lappend args $uhost $uport
+
+    set s [eval [linsert $args 0 tls_connect]]
 
     # record the tls connection status in the http state array.
     upvar state state
