@@ -12,8 +12,13 @@
 #                Eric Kemp-Benedict, february 2007
 # version 0.5:   added the population standard deviation and variance,
 #                as suggested by Dimitrios Zachariadis
+# version 0.6:   added pdf and cdf procedures for various distributions
+#                (provided by Eric Kemp-Benedict)
+# version 0.7:   added Kruskal-Wallis test (by Torsten Berg)
+# version 0.8:   added Wilcoxon test and Spearman rank correlation
 
-package provide math::statistics 0.5
+package provide math::statistics 0.8.0
+package require math
 
 # ::math::statistics --
 #   Namespace holding the procedures and variables
@@ -28,7 +33,9 @@ namespace eval ::math::statistics {
 	    test-normal lillieforsFit \
 	    autocorr crosscorr filter map samplescount median \
 	    test-2x2 print-2x2 control-xbar test_xbar \
-	    control-Rchart test-Rchart
+	    control-Rchart test-Rchart \
+	    test-Kruskal-Wallis analyse-Kruskal-Wallis group-rank \
+	    test-Wilcoxon spearman-rank spearman-rank-extended
     #
     # Error messages
     #
@@ -87,6 +94,12 @@ namespace eval ::math::statistics {
 proc ::math::statistics::BasicStats { type values } {
     variable TOOFEWDATA
 
+    if { [lsearch {all mean min max number stdev var pstdev pvar} $type] < 0 } {
+	return -code error \
+		-errorcode ARG -errorinfo [list unknown type of statistic -- $type] \
+		[list unknown type of statistic -- $type]
+    }
+
     set min    {}
     set max    {}
     set mean   {}
@@ -96,6 +109,7 @@ proc ::math::statistics::BasicStats { type values } {
     set sum    0.0
     set sumsq  0.0
     set number 0
+    set first  {}
 
     foreach value $values {
 	if { $value == {} } {
@@ -103,9 +117,13 @@ proc ::math::statistics::BasicStats { type values } {
 	}
 	set value [expr {double($value)}]
 
+	if { $first == {} } {
+	    set first $value
+	}
+
 	incr number
 	set  sum    [expr {$sum+$value}]
-	set  sumsq  [expr {$sumsq+$value*$value}]
+	set  sumsq  [expr {$sumsq+($value-$first)*($value-$first)}]
 
 	if { $min == {} || $value < $min } {
 	    set min $value
@@ -122,7 +140,7 @@ proc ::math::statistics::BasicStats { type values } {
     }
 
     if { $number > 1 } {
-	set var    [expr {($sumsq-$mean*$sum)/double($number-1)}]
+	set var    [expr {($sumsq-($mean-$first)*($sum-$number*$first))/double($number-1)}]
         #
         # Take care of a rare situation: uniform data might
         # cause a tiny negative difference
@@ -132,7 +150,7 @@ proc ::math::statistics::BasicStats { type values } {
         }
 	set stdev  [expr {sqrt($var)}]
     }
-	set pvar [expr {($sumsq-$mean*$sum)/double($number)}]
+	set pvar [expr {($sumsq-($mean-$first)*($sum-$number*$first))/double($number)}]
         #
         # Take care of a rare situation: uniform data might
         # cause a tiny negative difference
@@ -147,14 +165,7 @@ proc ::math::statistics::BasicStats { type values } {
     #
     # Return the appropriate value
     #
-    if { [lsearch {all mean min max number stdev var pstdev pvar} $type] >= 0 } {
-	# FRINK: nocheck
-	return [set $type]
-    } else {
-	return -code error \
-		-errorcode ARG -errorinfo [list unknown type of statistic -- $type] \
-		[list unknown type of statistic -- $type]
-    }
+    set $type
 }
 
 # histogram --
@@ -484,7 +495,6 @@ proc ::math::statistics::t-test-mean { data est_mean est_stdev confidence } {
 #
 proc ::math::statistics::interval-mean-stdev { data confidence } {
     variable TOOFEWDATA
-    variable student_t_table
 
     set allstats [BasicStats all $data]
 
@@ -495,9 +505,7 @@ proc ::math::statistics::interval-mean-stdev { data confidence } {
 
     if { $number > 1 } {
 	set degrees    [expr {$number-1}]
-	set student_t \
-		[::math::interpolation::interpolate2d $student_t_table \
-		$degrees $conf2]
+	set student_t  [expr {sqrt([Inverse-cdf-toms322 1 $degrees $conf2])}]
 	set mean_lower [expr {$mean-$student_t*$stdev/sqrt($number)}]
 	set mean_upper [expr {$mean+$student_t*$stdev/sqrt($number)}]
 	set stdev_lower {}
@@ -1281,6 +1289,8 @@ proc ::math::statistics::test-Rchart { control data } {
 
 
 
+
+
 #
 # Load the auxiliary scripts
 #
@@ -1288,6 +1298,8 @@ source [file join [file dirname [info script]] pdf_stat.tcl]
 source [file join [file dirname [info script]] plotstat.tcl]
 source [file join [file dirname [info script]] liststat.tcl]
 source [file join [file dirname [info script]] mvlinreg.tcl]
+source [file join [file dirname [info script]] kruskal.tcl]
+source [file join [file dirname [info script]] wilcoxon.tcl]
 
 #
 # Define the tables
