@@ -1,7 +1,7 @@
 # smtp.tcl - SMTP client
 #
 # Copyright (c) 1999-2000 Marshall T. Rose
-# Copyright (c) 2003-2005 Pat Thoyts
+# Copyright (c) 2003-2006 Pat Thoyts
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -32,7 +32,7 @@ catch {
 
 
 namespace eval ::smtp {
-    variable version 1.4.2
+    variable version 1.4.5
     variable trf 1
     variable smtp
     array set smtp { uid 0 }
@@ -92,6 +92,8 @@ if {[catch {package require Trf  2.0}]} {
 #                          request.
 #             -ports       A list of SMTP ports to use for each SMTP server
 #                          specified
+#             -client      The string to use as our host name for EHLO or HELO
+#                          This defaults to 'localhost' or [info hostname]
 #             -maxsecs     Maximum number of seconds to allow the SMTP server
 #                          to accept the message. If not specified, the default
 #                          is 120 seconds.
@@ -125,6 +127,7 @@ proc ::smtp::sendmessage {part args} {
     set originator ""
     set recipients ""
     set servers [list localhost]
+    set client "" ;# default is set after options processing
     set ports [list 25]
     set tlsP 1
     set tlspolicy {}
@@ -192,6 +195,10 @@ proc ::smtp::sendmessage {part args} {
                 set servers $value
             }
 
+            -client {
+                set client $value
+            }
+
             -ports {
                 set ports $value
             }
@@ -402,10 +409,12 @@ proc ::smtp::sendmessage {part args} {
         }
     }
 
-    if {![string compare $servers localhost]} {
-        set client localhost
-    } else {
-        set client [info hostname]
+    if {[string length $client] < 1} {
+        if {![string compare $servers localhost]} {
+            set client localhost
+        } else {
+            set client [info hostname]
+        }
     }
 
     # Create smtp token, which essentially means begin talking to the SMTP
@@ -677,6 +686,13 @@ proc ::smtp::initialize {args} {
     return -code $code -errorinfo $einfo -errorcode $ecode $result
 }
 
+# If we cannot load the tls package, ignore the error
+proc ::smtp::load_tls {} {
+    set r [catch {package require tls}]
+    if {$r} {set ::errorInfo ""}
+    return $r
+}
+
 proc ::smtp::initialize_ehlo {token} {
     global errorCode errorInfo
     upvar einfo einfo
@@ -730,7 +746,7 @@ proc ::smtp::initialize_ehlo {token} {
         if {($options(-usetls)) && ![info exists state(tls)] \
                 && (([lsearch $response(args) STARTTLS] >= 0)
                     || ([lsearch $response(args) TLS] >= 0))} {
-            if {![catch {package require tls}]} {
+            if {![load_tls]} {
                 set state(tls) 0
                 if {![catch {smtp::talk $token 300 STARTTLS} resp]} {
                     array set starttls $resp
@@ -1123,6 +1139,10 @@ proc ::smtp::wtextaux {token part} {
 	    while {[regsub -all -- {([^\r])\n} $result "\\1\r\n" result]} {}
             regsub -all -- {\n\.}      $result "\n.."   result
 
+            # Fix for bug #827436 - mail data must end with CRLF.CRLF
+            if {[string compare [string index $result end] "\n"] != 0} {
+                append result "\r\n"
+            }
             set state(size) [string length $result]
             puts -nonewline $state(sd) $result
             set result ""
