@@ -3,18 +3,19 @@
 #	URI parsing and fetch
 #
 # Copyright (c) 2000 Zveno Pty Ltd
+# Copyright (c) 2006 Pierre DAVID <Pierre.David@crc.u-strasbg.fr>
+# Copyright (c) 2006 Andreas Kupries <andreas_kupries@users.sourceforge.net>
 # Steve Ball, http://www.zveno.com/
 # Derived from urls.tcl by Andreas Kupries
 #
 # TODO:
 #	Handle www-url-encoding details
 #
-# CVS: $Id: uri.tcl,v 1.12 2002/01/18 20:51:17 andreas_kupries Exp $
+# CVS: $Id: uri.tcl,v 1.36 2011/03/23 04:39:54 andreas_kupries Exp $
 
 package require Tcl 8.2
-package provide uri 1.1
 
-namespace eval uri {
+namespace eval ::uri {
 
     namespace export split join
     namespace export resolve isrelative
@@ -23,7 +24,7 @@ namespace eval uri {
     namespace export register
 
     variable file:counter 0
-    
+
     # extend these variable in the coming namespaces
     variable schemes       {}
     variable schemePattern ""
@@ -48,11 +49,11 @@ namespace eval uri {
 	variable	hex		{[0-9A-Fa-f]}
 	variable	alphaDigit	{[A-Za-z0-9]}
 	variable	alphaDigitMinus	{[A-Za-z0-9-]}
-	
+
 	# next is <national | punctuation>
 	variable	unsafe		{[][<>"#%\{\}|\\^~`]} ;#" emacs hilit
 	variable	escape		"%${hex}${hex}"
-	
+
 	#	unreserved	= alpha | digit | safe | extra
 	#	xchar		= unreserved | reserved | escape
 
@@ -86,7 +87,7 @@ namespace eval uri {
 }
 
 
-# uri::register --
+# ::uri::register --
 #
 #	Register a scheme (and aliases) in the package. The command
 #	creates a namespace below "::uri" with the same name as the
@@ -105,7 +106,7 @@ namespace eval uri {
 # Results:
 #	None.
 
-proc uri::register {schemeList script} {
+proc ::uri::register {schemeList script} {
     variable schemes
     variable schemePattern
     variable url
@@ -136,10 +137,10 @@ proc uri::register {schemeList script} {
 
     # Now we can extend the variables which keep track of the registered schemes.
 
-    eval lappend schemes $schemeList
+    eval [linsert $schemeList 0 lappend schemes]
     set schemePattern	"([::join $schemes |]):"
 
-    foreach s schemeList {
+    foreach s $schemeList {
 	# FRINK: nocheck
 	set url2part($s) "${s}:[set ${scheme}::schemepart]"
 	# FRINK: nocheck
@@ -149,7 +150,7 @@ proc uri::register {schemeList script} {
     return
 }
 
-# uri::split --
+# ::uri::split --
 #
 #	Splits the given <a url> into its constituents.
 #
@@ -159,16 +160,16 @@ proc uri::register {schemeList script} {
 # Results:
 #	Tcl list containing constituents, suitable for 'array set'.
 
-proc uri::split {url} {
+proc ::uri::split {url {defaultscheme http}} {
 
     set url [string trim $url]
     set scheme {}
 
     # RFC 1738:	scheme = 1*[ lowalpha | digit | "+" | "-" | "." ]
-    regexp -- {^([a-z0-9+-.][a-z0-9+-.]*):} $url dummy scheme
+    regexp -- {^([a-z0-9+.-][a-z0-9+.-]*):} $url dummy scheme
 
     if {$scheme == {}} {
-	set scheme http
+	set scheme $defaultscheme
     }
 
     # ease maintenance: dynamic dispatch, able to handle all schemes
@@ -188,7 +189,7 @@ proc uri::split {url} {
     return [array get parts]
 }
 
-proc uri::SplitFtp {url} {
+proc ::uri::SplitFtp {url} {
     # @c Splits the given ftp-<a url> into its constituents.
     # @a url: The url to split, without! scheme specification.
     # @r List containing the constituents, suitable for 'array set'.
@@ -234,7 +235,7 @@ proc uri::SplitFtp {url} {
     return [array get parts]
 }
 
-proc uri::JoinFtp args {
+proc ::uri::JoinFtp args {
     array set components {
 	user {} pwd {} host {} port {}
 	path {} type {}
@@ -255,15 +256,15 @@ proc uri::JoinFtp args {
     if {[string length $components(type)]} {
 	set type \;type=$components(type)
     }
-    
+
     return ftp://${userPwd}$components(host)${port}/[string trimleft $components(path) /]$type
 }
 
-proc uri::SplitHttps {url} {
-    uri::SplitHttp $url
+proc ::uri::SplitHttps {url} {
+    return [SplitHttp $url]
 }
 
-proc uri::SplitHttp {url} {
+proc ::uri::SplitHttp {url} {
     # @c Splits the given http-<a url> into its constituents.
     # @a url: The url to split, without! scheme specification.
     # @r List containing the constituents, suitable for 'array set'.
@@ -287,23 +288,16 @@ proc uri::SplitHttp {url} {
     upvar #0 [namespace current]::http::search  search
     upvar #0 [namespace current]::http::segment segment
 
-    array set parts {host {} port {} path {} query {}}
+    array set parts {host {} port {} path {} query {} fragment {}}
 
     set searchPattern   "\\?(${search})\$"
     set fragmentPattern "#(${segment})\$"
 
-    # slash off possible query
+    # slash off possible fragment.
 
-    if {[regexp -indices -- $searchPattern $url match query]} {
-	set from [lindex $query 0]
-	set to   [lindex $query 1]
-
-	set parts(query) [string range $url $from $to]
-
-	set url [string replace $url [lindex $match 0] end]
-    }
-
-    # slash off possible fragment
+    # NOTE: This must be done before the query, because a fragment can
+    # follow a query, and slashing off the query first will take the
+    # fragment with it. Bug #3235340.
 
     if {[regexp -indices -- $fragmentPattern $url match fragment]} {
 	set from [lindex $fragment 0]
@@ -314,10 +308,23 @@ proc uri::SplitHttp {url} {
 	set url [string replace $url [lindex $match 0] end]
     }
 
+    # slash off possible query. the 'search' regexp, while official,
+    # is not good enough. We have apparently lots of urls in the wild
+    # which contain unquoted urls with queries in a query. The RE
+    # finds the embedded query, not the actual one. Using string first
+    # now instead of a RE
+
+    if {[set pos [string first ? $url]] >= 0} {
+	incr pos
+	set parts(query) [string range   $url $pos end]
+	incr pos -1
+	set url          [string replace $url $pos end]
+    }
+
     if {[string match "//*" $url]} {
 	set url [string range $url 2 end]
 
-	array set parts [GetHostPort url]
+	array set parts [GetUPHP url]
     }
 
     set parts(path) [string trimleft $url /]
@@ -325,18 +332,17 @@ proc uri::SplitHttp {url} {
     return [array get parts]
 }
 
-proc uri::JoinHttp {args} {
-    eval uri::JoinHttpInner http 80 $args
+proc ::uri::JoinHttp {args} {
+    return [eval [linsert $args 0 ::uri::JoinHttpInner http 80]]
 }
 
-proc uri::JoinHttps {args} {
-    eval uri::JoinHttpInner https 443 $args
+proc ::uri::JoinHttps {args} {
+    return [eval [linsert $args 0 ::uri::JoinHttpInner https 443]]
 }
 
-proc uri::JoinHttpInner {scheme defport args} {
-    array set components [list \
-	host {} port $defport path {} query {} \
-    ]
+proc ::uri::JoinHttpInner {scheme defport args} {
+    array set components {host {} path {} query {} fragment {}}
+    set       components(port) $defport
     array set components $args
 
     set port {}
@@ -351,10 +357,16 @@ proc uri::JoinHttpInner {scheme defport args} {
 
     regsub -- {^/} $components(path) {} components(path)
 
-    return $scheme://$components(host)$port/$components(path)$query
+    if { $components(fragment) != "" } {
+	set components(fragment) "#$components(fragment)"
+    } else {
+	set components(fragment) ""
+    }
+
+    return $scheme://$components(host)$port/$components(path)$query$components(fragment)
 }
 
-proc uri::SplitFile {url} {
+proc ::uri::SplitFile {url} {
     # @c Splits the given file-<a url> into its constituents.
     # @a url: The url to split, without! scheme specification.
     # @r List containing the constituents, suitable for 'array set'.
@@ -394,7 +406,7 @@ proc uri::SplitFile {url} {
     return [array get parts]
 }
 
-proc uri::JoinFile args {
+proc ::uri::JoinFile args {
     array set components {
 	host {} port {} path {}
     }
@@ -414,12 +426,12 @@ proc uri::JoinFile args {
     }
 }
 
-proc uri::SplitMailto {url} {
+proc ::uri::SplitMailto {url} {
     # @c Splits the given mailto-<a url> into its constituents.
     # @a url: The url to split, without! scheme specification.
     # @r List containing the constituents, suitable for 'array set'.
 
-    if {[regexp -- @ $url]} {
+    if {[string match "*@*" $url]} {
 	set url [::split $url @]
 	return [list user [lindex $url 0] host [lindex $url 1]]
     } else {
@@ -427,7 +439,7 @@ proc uri::SplitMailto {url} {
     }
 }
 
-proc uri::JoinMailto args {
+proc ::uri::JoinMailto args {
     array set components {
 	user {} host {}
     }
@@ -436,7 +448,99 @@ proc uri::JoinMailto args {
     return mailto:$components(user)@$components(host)
 }
 
-proc uri::GetUPHP {urlvar} {
+proc ::uri::SplitNews {url} {
+    if { [string first @ $url] >= 0 } {
+	return [list message-id $url]
+    } else {
+	return [list newsgroup-name $url]
+    }
+}
+
+proc ::uri::JoinNews args {
+    array set components {
+	message-id {} newsgroup-name {}
+    }
+    array set components $args
+    return news:$components(message-id)$components(newsgroup-name)
+}
+
+proc ::uri::SplitLdaps {url} {
+    ::uri::SplitLdap $url
+}
+
+proc ::uri::SplitLdap {url} {
+    # @c Splits the given Ldap-<a url> into its constituents.
+    # @a url: The url to split, without! scheme specification.
+    # @r List containing the constituents, suitable for 'array set'.
+
+    # general syntax:
+    # //<host>:<port>/<dn>?<attrs>?<scope>?<filter>?<extensions>
+    #
+    #   where <host> and <port> are as described in Section 5 of RFC 1738.
+    #   No user name or password is allowed.
+    #   If omitted, the port defaults to 389 for ldap, 636 for ldaps
+    #   <dn> is the base DN for the search
+    #   <attrs> is a comma separated list of attributes description
+    #   <scope> is either "base", "one" or "sub".
+    #   <filter> is a RFC 2254 filter specification
+    #   <extensions> are documented in RFC 2255
+    #
+
+    array set parts {host {} port {} dn {} attrs {} scope {} filter {} extensions {}}
+
+    #          host        port           dn          attrs       scope               filter     extns
+    set re {//([^:?/]+)(?::([0-9]+))?(?:/([^?]+)(?:\?([^?]*)(?:\?(base|one|sub)?(?:\?([^?]*)(?:\?(.*))?)?)?)?)?}
+
+    if {! [regexp $re $url match parts(host) parts(port) \
+		parts(dn) parts(attrs) parts(scope) parts(filter) \
+		parts(extensions)]} then {
+	return -code error "unable to match URL \"$url\""
+    }
+
+    set parts(attrs) [::split $parts(attrs) ","]
+
+    return [array get parts]
+}
+
+proc ::uri::JoinLdap {args} {
+    return [eval [linsert $args 0 ::uri::JoinLdapInner ldap 389]]
+}
+
+proc ::uri::JoinLdaps {args} {
+    return [eval [linsert $args 0 ::uri::JoinLdapInner ldaps 636]]
+}
+
+proc ::uri::JoinLdapInner {scheme defport args} {
+    array set components {host {} port {} dn {} attrs {} scope {} filter {} extensions {}}
+    set       components(port) $defport
+    array set components $args
+
+    set port {}
+    if {[string length $components(port)] && $components(port) != $defport} {
+	set port :$components(port)
+    }
+
+    set url "$scheme://$components(host)$port"
+
+    set components(attrs) [::join $components(attrs) ","]
+
+    set s ""
+    foreach c {dn attrs scope filter extensions} {
+	if {[string equal $c "dn"]} then {
+	    append s "/"
+	} else {
+	    append s "?"
+	}
+	if {! [string equal $components($c) ""]} then {
+	    append url "${s}$components($c)"
+	    set s ""
+	}
+    }
+
+    return $url
+}
+
+proc ::uri::GetUPHP {urlvar} {
     # @c Parse user, password host and port out of the url stored in
     # @c variable <a urlvar>.
     # @d Side effect: The extracted information is removed from the given url.
@@ -463,7 +567,7 @@ proc uri::GetUPHP {urlvar} {
     if {[regexp -indices -- $upPattern $url match theUser c d thePassword]} {
 	set fu	[lindex $theUser 0]
 	set tu	[lindex $theUser 1]
-	    
+
 	set fp	[lindex $thePassword 0]
 	set tp	[lindex $thePassword 1]
 
@@ -497,7 +601,7 @@ proc uri::GetUPHP {urlvar} {
     return [array get parts]
 }
 
-proc uri::GetHostPort {urlvar} {
+proc ::uri::GetHostPort {urlvar} {
     # @c Parse host and port out of the url stored in variable <a urlvar>.
     # @d Side effect: The extracted information is removed from the given url.
     # @r List containing the extracted information in a format suitable for
@@ -531,7 +635,7 @@ proc uri::GetHostPort {urlvar} {
     return [array get parts]
 }
 
-# uri::resolve --
+# ::uri::resolve --
 #
 #	Resolve an arbitrary URL, given a base URL
 #
@@ -542,7 +646,7 @@ proc uri::GetHostPort {urlvar} {
 # Results:
 #	Returns a URL
 
-proc uri::resolve {base url} {
+proc ::uri::resolve {base url} {
     if {[string length $url]} {
 	if {[isrelative $url]} {
 
@@ -553,20 +657,21 @@ proc uri::resolve {base url} {
 		https -
 		ftp -
 		file {
-		    if {[string match /* $url]} {
-			array set relparts [split "$baseparts(scheme)://$url"]
-			array set baseparts [list path $relparts(path)]
-			catch {array set baseparts [list query $relparts(query)]}
-			return [eval join [array get baseparts]]
-		    } elseif {[string match */ $baseparts(path)]} {
-			return ${base}$url
+		    array set relparts [split $url]
+		    if { [string match /* $url] } {
+			catch { set baseparts(path) $relparts(path) }
+		    } elseif { [string match */ $baseparts(path)] } {
+			set baseparts(path) "$baseparts(path)$relparts(path)"
 		    } else {
-			set path [lreplace [::split $baseparts(path) /] end end]
-			array set baseparts [list path [::join $path /]/$url]
-			return [eval join [array get baseparts]]
+			if { [string length $relparts(path)] > 0 } {
+			    set path [lreplace [::split $baseparts(path) /] end end]
+			    set baseparts(path) "[::join $path /]/$relparts(path)"
+			}
 		    }
+		    catch { set baseparts(query) $relparts(query) }
+		    catch { set baseparts(fragment) $relparts(fragment) }
+            return [eval [linsert [array get baseparts] 0 join]]
 		}
-
 		default {
 		    return -code error "unable to resolve relative URL \"$url\""
 		}
@@ -580,7 +685,7 @@ proc uri::resolve {base url} {
     }
 }
 
-# isrelative --
+# ::uri::isrelative --
 #
 #	Determines whether a URL is absolute or relative
 #
@@ -590,11 +695,11 @@ proc uri::resolve {base url} {
 # Results:
 #	Returns 1 if the URL is relative, 0 otherwise
 
-proc uri::isrelative url {
+proc ::uri::isrelative url {
     return [expr {![regexp -- {^[a-z0-9+-.][a-z0-9+-.]*:} $url]}]
 }
 
-# geturl --
+# ::uri::geturl --
 #
 #	Fetch the data from an arbitrary URL.
 #
@@ -608,12 +713,12 @@ proc uri::isrelative url {
 # Results:
 #	Depends on scheme
 
-proc uri::geturl {url args} {
+proc ::uri::geturl {url args} {
     array set urlparts [split $url]
 
     switch -- $urlparts(scheme) {
 	file {
-	    return [eval file_geturl [list $url] $args]
+        return [eval [linsert $args 0 file_geturl $url]]
 	}
 	default {
 	    # Load a geturl package for the scheme first and only if
@@ -622,12 +727,12 @@ proc uri::geturl {url args} {
 	    if {[catch {package require $urlparts(scheme)::geturl}]} {
 		package require $urlparts(scheme)
 	    }
-	    return [eval [list $urlparts(scheme)::geturl $url] $args]
+        return [eval [linsert $args 0 $urlparts(scheme)::geturl $url]]
 	}
     }
 }
 
-# uri::file_geturl --
+# ::uri::file_geturl --
 #
 #	geturl implementation for file: scheme
 #
@@ -642,7 +747,7 @@ proc uri::geturl {url args} {
 # Results:
 #	Returns data from file
 
-proc uri::file_geturl {url args} {
+proc ::uri::file_geturl {url args} {
     variable file:counter
 
     set var [namespace current]::file[incr file:counter]
@@ -661,7 +766,7 @@ proc uri::file_geturl {url args} {
     return $var
 }
 
-# uri::join --
+# ::uri::join --
 #
 #	Format a URL
 #
@@ -671,18 +776,18 @@ proc uri::file_geturl {url args} {
 # Results:
 #	A URL
 
-proc uri::join args {
+proc ::uri::join args {
     array set components $args
 
-    return [eval [list Join[string totitle $components(scheme)]] $args]
+    return [eval [linsert $args 0 Join[string totitle $components(scheme)]]]
 }
 
-# uri::canonicalize --
+# ::uri::canonicalize --
 #
 #	Canonicalize a URL
 #
 # Acknowledgements:
-#	Andreas Kupries, a.kupries@westend.com
+#	Andreas Kupries <andreas_kupries@users.sourceforge.net>
 #
 # Arguments:
 #	uri	URI (which contains a path component)
@@ -690,26 +795,50 @@ proc uri::join args {
 # Results:
 #	The canonical form of the URI
 
-proc uri::canonicalize uri {
+proc ::uri::canonicalize uri {
 
     # Make uri canonical with respect to dots (path changing commands)
     #
     # Remove single dots (.)  => pwd not changing
     # Remove double dots (..) => gobble previous segment of path
+    #
+    # Fixes for this command:
+    #
+    # * Ignore any url which cannot be split into components by this
+    #   module. Just assume that such urls do not have a path to
+    #   canonicalize.
+    #
+    # * Ignore any url which could be split into components, but does
+    #   not have a path component.
+    #
+    # In the text above 'ignore' means
+    # 'return the url unchanged to the caller'.
 
-    while {[regexp -- {/\./} $uri]} {
-        regsub -all -- {/\./} $uri {/} uri
+    if {[catch {array set u [::uri::split $uri]}]} {
+	return $uri
     }
-    while {[regexp -- {/\.\./} $uri]} {
-	if {![regsub -- {/[^./]*/\.\./} $uri {/} uri]} {
-	    # The regexp found 'foo://bar.com/../baz', but this
-	    # cannot be handled by the regsub. Simply remove the
-	    # dots, as is done for the singles to break out of
-	    # infinity.
-	    regsub -- {/\.\./} $uri {/} uri
-	}
+    if {![info exists u(path)]} {
+	return $uri
     }
 
+    set uri $u(path)
+
+    # Remove leading "./" "../" "/.." (and "/../")
+    regsub -all -- {^(\./)+}    $uri {}  uri
+    regsub -all -- {^/(\.\./)+} $uri {/} uri
+    regsub -all -- {^(\.\./)+}  $uri {}  uri
+
+    # Remove inner /./ and /../
+    while {[regsub -all -- {/\./}         $uri {/} uri]} {}
+    while {[regsub -all -- {/[^/]+/\.\./} $uri {/} uri]} {}
+    while {[regsub -all -- {^[^/]+/\.\./} $uri {}  uri]} {}
+    # Munge trailing /..
+    while {[regsub -all -- {/[^/]+/\.\.} $uri {/} uri]} {}
+    if { $uri == ".." } { set uri "/" }
+
+    set u(path) $uri
+    set uri [eval [linsert [array get u] 0 ::uri::join]]
+
     return $uri
 }
 
@@ -750,12 +879,19 @@ proc uri::canonicalize uri {
 #		message-id/content-id
 # cid	content-id
 # ------------------------------------------------
+#
+# (RFC 2255)
+# ------------------------------------------------
+# scheme	basic syntax of scheme specific part
+# ------------------------------------------------
+# ldap		//<host>:<port>/<dn>?<attrs>?<scope>?<filter>?<extensions>
+# ------------------------------------------------
 
 # FTP
 uri::register ftp {
-    set escape [set [namespace parent [namespace current]]::basic::escape]
-    set login  [set [namespace parent [namespace current]]::basic::login]
-    
+    variable escape [set [namespace parent [namespace current]]::basic::escape]
+    variable login  [set [namespace parent [namespace current]]::basic::login]
+
     variable	charN	{[a-zA-Z0-9$_.+!*'(,)?:@&=-]}
     variable	char	"(${charN}|${escape})"
     variable	segment	"${char}*"
@@ -771,8 +907,8 @@ uri::register ftp {
 
 # FILE
 uri::register file {
-    set host [set [namespace parent [namespace current]]::basic::host]
-    set path [set [namespace parent [namespace current]]::ftp::path]
+    variable	host [set [namespace parent [namespace current]]::basic::host]
+    variable	path [set [namespace parent [namespace current]]::ftp::path]
 
     variable	schemepart	"//(${host}|localhost)?/${path}"
     variable	url		"file:${schemepart}"
@@ -780,8 +916,10 @@ uri::register file {
 
 # HTTP
 uri::register http {
-    set escape		[set [namespace parent [namespace current]]::basic::escape]
-    set hostOrPort	[set [namespace parent [namespace current]]::basic::hostOrPort]
+    variable	escape \
+        [set [namespace parent [namespace current]]::basic::escape]
+    variable	hostOrPort	\
+        [set [namespace parent [namespace current]]::basic::hostOrPort]
 
     variable	charN		{[a-zA-Z0-9$_.+!*'(,);:@&=-]}
     variable	char		"($charN|${escape})"
@@ -797,9 +935,12 @@ uri::register http {
 
 # GOPHER
 uri::register gopher {
-    set xChar		[set [namespace parent [namespace current]]::basic::xChar]
-    set hostOrPort	[set [namespace parent [namespace current]]::basic::hostOrPort]
-    set search		[set [namespace parent [namespace current]]::http::search]
+    variable	xChar \
+        [set [namespace parent [namespace current]]::basic::xChar]
+    variable	hostOrPort \
+        [set [namespace parent [namespace current]]::basic::hostOrPort]
+    variable	search \
+        [set [namespace parent [namespace current]]::http::search]
 
     variable	type		$xChar
     variable	selector	"$xChar*"
@@ -808,36 +949,39 @@ uri::register gopher {
 	    "//${hostOrPort}(/(${type}(${selector}(%09${search}(%09${string})?)?)?)?)?"
     variable	url		"gopher:${schemepart}"
 }
-	
+
 # MAILTO
 uri::register mailto {
-    set xChar	[set [namespace parent [namespace current]]::basic::xChar]
-    set host	[set [namespace parent [namespace current]]::basic::host]
+    variable xChar [set [namespace parent [namespace current]]::basic::xChar]
+    variable host  [set [namespace parent [namespace current]]::basic::host]
 
-    variable	schemepart	"$xChar+(@${host})?"
-    variable	url		"mailto:${schemepart}"
+    variable schemepart	"$xChar+(@${host})?"
+    variable url	"mailto:${schemepart}"
 }
 
 # NEWS
 uri::register news {
-    set escape		[set [namespace parent [namespace current]]::basic::escape]
-    set alpha		[set [namespace parent [namespace current]]::basic::alpha]
-    set host		[set [namespace parent [namespace current]]::basic::host]
+    variable escape [set [namespace parent [namespace current]]::basic::escape]
+    variable alpha  [set [namespace parent [namespace current]]::basic::alpha]
+    variable host   [set [namespace parent [namespace current]]::basic::host]
 
     variable	aCharN		{[a-zA-Z0-9$_.+!*'(,);/?:&=-]}
     variable	aChar		"($aCharN|${escape})"
     variable	gChar		{[a-zA-Z0-9$_.+-]}
-    variable	group		"${alpha}${gChar}*"
-    variable	article		"${aChar}+@${host}"
-    variable	schemepart	"\\*|${group}|${article}"
+    variable	newsgroup-name	"${alpha}${gChar}*"
+    variable	message-id	"${aChar}+@${host}"
+    variable	schemepart	"\\*|${newsgroup-name}|${message-id}"
     variable	url		"news:${schemepart}"
 }
 
 # WAIS
 uri::register wais {
-    set uChar		[set [namespace parent [namespace current]]::basic::xChar]
-    set hostOrPort	[set [namespace parent [namespace current]]::basic::hostOrPort]
-    set search		[set [namespace parent [namespace current]]::http::search]
+    variable	uChar \
+        [set [namespace parent [namespace current]]::basic::xChar]
+    variable	hostOrPort \
+        [set [namespace parent [namespace current]]::basic::hostOrPort]
+    variable	search \
+        [set [namespace parent [namespace current]]::http::search]
 
     variable	db		"${uChar}*"
     variable	type		"${uChar}*"
@@ -849,7 +993,7 @@ uri::register wais {
 
     #variable	schemepart	"${doc}|${index}|${database}"
 
-    variable schemepart \
+    variable	schemepart \
 	    "//${hostOrPort}/${db}((\\?${search})|(/${type}/${path}))?"
 
     variable	url		"wais:${schemepart}"
@@ -857,9 +1001,12 @@ uri::register wais {
 
 # PROSPERO
 uri::register prospero {
-    set escape		[set [namespace parent [namespace current]]::basic::escape]
-    set hostOrPort	[set [namespace parent [namespace current]]::basic::hostOrPort]
-    set path		[set [namespace parent [namespace current]]::ftp::path]
+    variable	escape \
+        [set [namespace parent [namespace current]]::basic::escape]
+    variable	hostOrPort \
+        [set [namespace parent [namespace current]]::basic::hostOrPort]
+    variable	path \
+        [set [namespace parent [namespace current]]::ftp::path]
 
     variable	charN		{[a-zA-Z0-9$_.+!*'(,)?:@&-]}
     variable	char		"(${charN}|$escape)"
@@ -871,3 +1018,21 @@ uri::register prospero {
     variable	schemepart	"//${hostOrPort}/${path}(${fieldspec})*"
     variable	url		"prospero:$schemepart"
 }
+
+# LDAP
+uri::register ldap {
+    variable	hostOrPort \
+        [set [namespace parent [namespace current]]::basic::hostOrPort]
+
+    # very crude parsing
+    variable	dn		{[^?]*}
+    variable	attrs		{[^?]*}
+    variable	scope		"base|one|sub"
+    variable	filter		{[^?]*}
+    # extensions are not handled yet
+
+    variable	schemepart	"//${hostOrPort}(/${dn}(\?${attrs}(\?(${scope})(\?${filter})?)?)?)?"
+    variable	url		"ldap:$schemepart"
+}
+
+package provide uri 1.2.2
